# PYZ, EXE and COLLECT referenced in https://pyinstaller.org/en/stable/spec-files.html#spec-file-operation
# MERGE is referenced in https://pyinstaller.org/en/stable/spec-files.html#example-merge-spec-file
# hide_console referenced in https://pyinstaller.org/en/stable/feature-notes.html#automatic-hiding-and-minimization-of-console-window-under-windows
# Not to be imported during runtime, but is the type reference for spec files which are executed as python code
import sys
from _typeshed import FileDescriptorOrPath, StrOrBytesPath, StrPath, Unused
from collections.abc import Iterable, Mapping, Sequence
from types import CodeType
from typing import ClassVar, Final, Literal
from typing_extensions import TypeAlias

from PyInstaller.building import _PyiBlockCipher
from PyInstaller.building.build_main import Analysis
from PyInstaller.building.datastruct import Target, _TOCTuple
from PyInstaller.building.splash import Splash
from PyInstaller.utils.win32.versioninfo import VSVersionInfo

if sys.platform == "darwin":
    _TargetArch: TypeAlias = Literal["x86_64", "arm64", "universal2"]
    _SupportedTargetArchParam: TypeAlias = _TargetArch | None
    _CodesignIdentity: TypeAlias = str | None
    _CodesignIdentityParam: TypeAlias = str | None
else:
    _TargetArch: TypeAlias = None
    _SupportedTargetArchParam: TypeAlias = Unused
    _CodesignIdentity: TypeAlias = None
    _CodesignIdentityParam: TypeAlias = Unused

if sys.platform == "win32":
    _Icon: TypeAlias = list[StrPath] | str
    _IconParam: TypeAlias = StrPath | list[StrPath] | None
elif sys.platform == "darwin":
    _Icon: TypeAlias = list[StrPath] | None
    _IconParam: TypeAlias = StrPath | list[StrPath] | None
else:
    _Icon: TypeAlias = None
    _IconParam: TypeAlias = Unused

if sys.platform == "win32":
    _VersionSrc: TypeAlias = VSVersionInfo | None
    _VersionParam: TypeAlias = VSVersionInfo | StrOrBytesPath | None
    _Manifest: TypeAlias = bytes
    _ManifestParam: TypeAlias = str | None
else:
    _VersionSrc: TypeAlias = None
    _VersionParam: TypeAlias = Unused
    _Manifest: TypeAlias = None
    _ManifestParam: TypeAlias = Unused

_HideConsole: TypeAlias = Literal["hide-early", "minimize-early", "hide-late", "minimize-late"] | None

class PYZ(Target):
    name: str
    cipher: _PyiBlockCipher
    dependencies: list[_TOCTuple]
    toc: list[_TOCTuple]
    code_dict: dict[str, CodeType]
    def __init__(self, *tocs: Iterable[_TOCTuple], name: str | None = None, cipher: _PyiBlockCipher = None) -> None: ...
    def assemble(self) -> None: ...

class PKG(Target):
    xformdict: ClassVar[dict[str, str]]
    toc: list[_TOCTuple]
    cdict: Mapping[str, bool]
    python_lib_name: str
    name: str
    exclude_binaries: bool
    strip_binaries: bool
    upx_binaries: bool
    upx_exclude: Iterable[str]
    target_arch: _TargetArch | None
    codesign_identity: _CodesignIdentity
    entitlements_file: FileDescriptorOrPath | None
    def __init__(
        self,
        toc: Iterable[_TOCTuple],
        python_lib_name: str,
        name: str | None = None,
        cdict: Mapping[str, bool] | None = None,
        exclude_binaries: bool = False,
        strip_binaries: bool = False,
        upx_binaries: bool = False,
        upx_exclude: Iterable[str] | None = None,
        target_arch: _SupportedTargetArchParam = None,
        codesign_identity: _CodesignIdentityParam = None,
        entitlements_file: FileDescriptorOrPath | None = None,
    ) -> None: ...
    def assemble(self) -> None: ...

class EXE(Target):
    exclude_binaries: bool
    bootloader_ignore_signals: bool
    console: bool
    hide_console: _HideConsole
    disable_windowed_traceback: bool
    debug: bool
    name: str
    icon: _Icon
    versrsrc: _VersionSrc
    manifest: _Manifest
    embed_manifest: bool
    resources: Sequence[str]
    strip: bool
    upx_exclude: Iterable[str]
    runtime_tmpdir: str | None
    contents_directory: str | None
    append_pkg: bool
    uac_admin: bool
    uac_uiaccess: bool
    argv_emulation: bool
    target_arch: _TargetArch
    codesign_identity: _CodesignIdentity
    entitlements_file: FileDescriptorOrPath | None
    upx: bool
    pkgname: str
    toc: list[_TOCTuple]
    pkg: PKG
    dependencies: list[_TOCTuple]
    exefiles: list[_TOCTuple]
    def __init__(
        self,
        *args: Iterable[_TOCTuple] | PYZ | Splash,
        exclude_binaries: bool = False,
        bootloader_ignore_signals: bool = False,
        console: bool = True,
        hide_console: _HideConsole = None,
        disable_windowed_traceback: bool = False,
        debug: bool = False,
        name: str | None = None,
        icon: _IconParam = None,
        version: _VersionParam = None,
        manifest: _ManifestParam = None,
        embed_manifest: Literal[True] = True,
        resources: Sequence[str] = ...,
        strip: bool = False,
        upx_exclude: Iterable[str] = ...,
        runtime_tmpdir: str | None = None,
        contents_directory: str = "_internal",
        append_pkg: bool = True,
        uac_admin: bool = False,
        uac_uiaccess: bool = False,
        argv_emulation: bool = False,
        target_arch: _SupportedTargetArchParam = None,
        codesign_identity: _CodesignIdentityParam = None,
        entitlements_file: FileDescriptorOrPath | None = None,
        upx: bool = False,
        cdict: Mapping[str, bool] | None = None,
    ) -> None: ...
    mtm: float
    def assemble(self) -> None: ...

class COLLECT(Target):
    strip_binaries: bool
    upx_exclude: Iterable[str]
    console: bool
    target_arch: _TargetArch | None
    codesign_identity: _CodesignIdentity
    entitlements_file: FileDescriptorOrPath | None
    upx_binaries: bool
    name: str
    toc: list[_TOCTuple]
    def __init__(
        self,
        *args: Iterable[_TOCTuple] | EXE,
        strip: bool = False,
        upx_exclude: Iterable[str] = ...,
        upx: bool = False,
        name: str,
    ) -> None: ...
    def assemble(self) -> None: ...

class MERGE:
    def __init__(self, *args: tuple[Analysis, Unused, str]) -> None: ...

UNCOMPRESSED: Final = False
COMPRESSED: Final = True



from collections.abc import Callable, Collection, Generator
from typing import Any
from typing_extensions import TypeAlias

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = [
    "dijkstra_path",
    "dijkstra_path_length",
    "bidirectional_dijkstra",
    "single_source_dijkstra",
    "single_source_dijkstra_path",
    "single_source_dijkstra_path_length",
    "multi_source_dijkstra",
    "multi_source_dijkstra_path",
    "multi_source_dijkstra_path_length",
    "all_pairs_dijkstra",
    "all_pairs_dijkstra_path",
    "all_pairs_dijkstra_path_length",
    "dijkstra_predecessor_and_distance",
    "bellman_ford_path",
    "bellman_ford_path_length",
    "single_source_bellman_ford",
    "single_source_bellman_ford_path",
    "single_source_bellman_ford_path_length",
    "all_pairs_bellman_ford_path",
    "all_pairs_bellman_ford_path_length",
    "bellman_ford_predecessor_and_distance",
    "negative_edge_cycle",
    "find_negative_cycle",
    "goldberg_radzik",
    "johnson",
]

_WeightFunc: TypeAlias = Callable[
    [_Node, _Node, dict[str, Any]],  # Any: type of edge data cannot be known statically
    float | None,  # the weight or None to indicate a hidden edge
]

@_dispatchable
def dijkstra_path(
    G: Graph[_Node], source: _Node, target: _Node, weight: str | _WeightFunc[_Node] | None = "weight"
) -> list[_Node]: ...
@_dispatchable
def dijkstra_path_length(
    G: Graph[_Node], source: _Node, target: _Node, weight: str | _WeightFunc[_Node] | None = "weight"
) -> float: ...
@_dispatchable
def single_source_dijkstra_path(
    G: Graph[_Node], source: _Node, cutoff: float | None = None, weight: str | _WeightFunc[_Node] | None = "weight"
) -> dict[_Node, list[_Node]]: ...
@_dispatchable
def single_source_dijkstra_path_length(
    G: Graph[_Node], source: _Node, cutoff: float | None = None, weight: str | _WeightFunc[_Node] | None = "weight"
) -> dict[_Node, float]: ...
@_dispatchable
def single_source_dijkstra(
    G: Graph[_Node],
    source: _Node,
    target: _Node | None = None,
    cutoff: float | None = None,
    weight: str | _WeightFunc[_Node] | None = "weight",
) -> tuple[dict[_Node, float], dict[_Node, list[_Node]]] | tuple[float, list[_Node]]: ...  # TODO: overload on target
@_dispatchable
def multi_source_dijkstra_path(
    G: Graph[_Node], sources: Collection[_Node], cutoff: float | None = None, weight: str | _WeightFunc[_Node] | None = "weight"
) -> dict[_Node, list[_Node]]: ...
@_dispatchable
def multi_source_dijkstra_path_length(
    G: Graph[_Node], sources: Collection[_Node], cutoff: float | None = None, weight: str | _WeightFunc[_Node] | None = "weight"
) -> dict[_Node, float]: ...
@_dispatchable
def multi_source_dijkstra(
    G: Graph[_Node],
    sources: Collection[_Node],
    target: _Node | None = None,
    cutoff: float | None = None,
    weight: str | _WeightFunc[_Node] | None = "weight",
) -> tuple[dict[_Node, float], dict[_Node, list[_Node]]] | tuple[float, list[_Node]]: ...  # TODO: overload on target
@_dispatchable
def dijkstra_predecessor_and_distance(
    G: Graph[_Node], source: _Node, cutoff: float | None = None, weight: str | _WeightFunc[_Node] | None = "weight"
) -> tuple[dict[_Node, list[_Node]], dict[_Node, float]]: ...
@_dispatchable
def all_pairs_dijkstra(
    G: Graph[_Node], cutoff: float | None = None, weight: str | _WeightFunc[_Node] | None = "weight"
) -> Generator[tuple[_Node, tuple[dict[_Node, float], dict[_Node, list[_Node]]]]]: ...
@_dispatchable
def all_pairs_dijkstra_path_length(
    G: Graph[_Node], cutoff: float | None = None, weight: str | _WeightFunc[_Node] | None = "weight"
) -> Generator[tuple[_Node, dict[_Node, float]]]: ...
@_dispatchable
def all_pairs_dijkstra_path(
    G: Graph[_Node], cutoff: float | None = None, weight: str | _WeightFunc[_Node] | None = "weight"
) -> Generator[tuple[_Node, dict[_Node, list[_Node]]]]: ...
@_dispatchable
def bellman_ford_predecessor_and_distance(
    G: Graph[_Node],
    source: _Node,
    target: _Node | None = None,
    weight: str | _WeightFunc[_Node] | None = "weight",
    heuristic: bool = False,
) -> tuple[dict[_Node, list[_Node]], dict[_Node, float]]: ...
@_dispatchable
def bellman_ford_path(
    G: Graph[_Node], source: _Node, target: _Node, weight: str | _WeightFunc[_Node] | None = "weight"
) -> list[_Node]: ...
@_dispatchable
def bellman_ford_path_length(
    G: Graph[_Node], source: _Node, target: _Node, weight: str | _WeightFunc[_Node] | None = "weight"
) -> float: ...
@_dispatchable
def single_source_bellman_ford_path(
    G: Graph[_Node], source: _Node, weight: str | _WeightFunc[_Node] | None = "weight"
) -> dict[_Node, list[_Node]]: ...
@_dispatchable
def single_source_bellman_ford_path_length(
    G: Graph[_Node], source: _Node, weight: str | _WeightFunc[_Node] | None = "weight"
) -> dict[_Node, float]: ...
@_dispatchable
def single_source_bellman_ford(
    G: Graph[_Node], source: _Node, target: _Node | None = None, weight: str | _WeightFunc[_Node] | None = "weight"
) -> tuple[dict[_Node, float], dict[_Node, list[_Node]]] | tuple[float, list[_Node]]: ...  # TODO: overload on target
@_dispatchable
def all_pairs_bellman_ford_path_length(
    G: Graph[_Node], weight: str | _WeightFunc[_Node] | None = "weight"
) -> Generator[tuple[_Node, dict[_Node, float]]]: ...
@_dispatchable
def all_pairs_bellman_ford_path(
    G: Graph[_Node], weight: str | _WeightFunc[_Node] | None = "weight"
) -> Generator[tuple[_Node, dict[_Node, list[_Node]]]]: ...
@_dispatchable
def goldberg_radzik(
    G: Graph[_Node], source: _Node, weight: str | _WeightFunc[_Node] | None = "weight"
) -> tuple[dict[_Node, _Node | None], dict[_Node, float]]: ...
@_dispatchable
def negative_edge_cycle(G: Graph[_Node], weight: str | _WeightFunc[_Node] | None = "weight", heuristic: bool = True) -> bool: ...
@_dispatchable
def find_negative_cycle(G: Graph[_Node], source: _Node, weight: str | _WeightFunc[_Node] | None = "weight") -> list[_Node]: ...
@_dispatchable
def bidirectional_dijkstra(
    G: Graph[_Node], source: _Node, target: _Node, weight: str | _WeightFunc[_Node] | None = "weight"
) -> tuple[float, list[_Node]]: ...
@_dispatchable
def johnson(G: Graph[_Node], weight: str | _WeightFunc[_Node] | None = "weight") -> dict[_Node, dict[_Node, list[_Node]]]: ...



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from numpy.random import RandomState

__all__ = ["spanner"]

@_dispatchable
def spanner(G: Graph[_Node], stretch: float, weight: str | None = None, seed: int | RandomState | None = None): ...



from typing_extensions import assert_type

import networkx as nx
from networkx.classes.reportviews import DegreeView, DiDegreeView

# Test covariant dict type for `pos` in nx_latex functions
G: "nx.Graph[int]" = nx.Graph([(1, 2), (2, 3), (3, 4)])
nx.to_latex_raw(G, pos=nx.spring_layout(G, seed=42))  # OK: dict[node, ndarray]
pos1: dict[int, tuple[int, int]] = {1: (1, 2), 2: (3, 4), 3: (5, 6), 4: (7, 8)}
nx.to_latex_raw(G, pos=pos1)  # OK: dict[node, 2-tuple]
pos2: dict[int, str] = {1: "(1, 2)", 2: "(3, 4)", 3: "(5, 6)", 4: "(7, 8)"}
nx.to_latex_raw(G, pos=pos2)  # OK: dict[node, str]
pos3: dict[int, int] = {1: 1, 2: 3, 3: 5, 4: 7}
nx.to_latex_raw(G, pos=pos3)  # type: ignore # dict keys must be str or collection

# Test that we don't confuse str and Iterable[str] in DiDegreeView.__call__
G_str = nx.Graph[str]()
di_degree_view = DiDegreeView(G_str)
assert_type(di_degree_view(""), int)
assert_type(di_degree_view([""]), DiDegreeView[str])
assert_type(di_degree_view({""}), DiDegreeView[str])
degree_view = DegreeView(G_str)
assert_type(degree_view(""), int)
assert_type(degree_view([""]), DegreeView[str])
assert_type(degree_view({""}), DegreeView[str])



import argparse

def main_inner(parser, argns): ...

class HelpFormatter(argparse.HelpFormatter):
    def __init__(self, prog, indent_increment: int = 2, max_help_position: int = 16, width=None) -> None: ...

def main(args=...): ...



from typing import Final

XK_blank: Final = 0x9DF
XK_soliddiamond: Final = 0x9E0
XK_checkerboard: Final = 0x9E1
XK_ht: Final = 0x9E2
XK_ff: Final = 0x9E3
XK_cr: Final = 0x9E4
XK_lf: Final = 0x9E5
XK_nl: Final = 0x9E8
XK_vt: Final = 0x9E9
XK_lowrightcorner: Final = 0x9EA
XK_uprightcorner: Final = 0x9EB
XK_upleftcorner: Final = 0x9EC
XK_lowleftcorner: Final = 0x9ED
XK_crossinglines: Final = 0x9EE
XK_horizlinescan1: Final = 0x9EF
XK_horizlinescan3: Final = 0x9F0
XK_horizlinescan5: Final = 0x9F1
XK_horizlinescan7: Final = 0x9F2
XK_horizlinescan9: Final = 0x9F3
XK_leftt: Final = 0x9F4
XK_rightt: Final = 0x9F5
XK_bott: Final = 0x9F6
XK_topt: Final = 0x9F7
XK_vertbar: Final = 0x9F8



from collections.abc import Iterable, Iterator
from typing import Final, Literal, TypeVar, overload, type_check_only
from typing_extensions import Self, TypeAlias

_ColorT = TypeVar("_ColorT", bound=Color)
# NOTE: Reportlab is very inconsistent and sometimes uses the interpretation
#       used in reportlab.pdfgen.textobject instead, so we pick a different name
_ConvertibleToColor: TypeAlias = Color | list[float] | tuple[float, float, float, float] | tuple[float, float, float] | str | int

__version__: Final[str]

class Color:
    red: float
    green: float
    blue: float
    alpha: float
    def __init__(self, red: float = 0, green: float = 0, blue: float = 0, alpha: float = 1) -> None: ...
    @property
    def __key__(self) -> tuple[float, ...]: ...
    def __hash__(self) -> int: ...
    def __comparable__(self, other: object) -> bool: ...
    def __lt__(self, other: object) -> bool: ...
    def __eq__(self, other: object) -> bool: ...
    def __le__(self, other: object) -> bool: ...
    def __gt__(self, other: object) -> bool: ...
    def __ge__(self, other: object) -> bool: ...
    def rgb(self) -> tuple[float, float, float]: ...
    def rgba(self) -> tuple[float, float, float, float]: ...
    def bitmap_rgb(self) -> tuple[int, int, int]: ...
    def bitmap_rgba(self) -> tuple[int, int, int, int]: ...
    def hexval(self) -> str: ...
    def hexvala(self) -> str: ...
    def int_rgb(self) -> int: ...
    def int_rgba(self) -> int: ...
    def int_argb(self) -> int: ...
    @property
    def cKwds(self) -> Iterator[tuple[str, int]]: ...
    # NOTE: Possible arguments depend on __init__, so this violates LSP
    #       For now we just leave it unchecked
    def clone(self, **kwds) -> Self: ...
    @property
    def normalizedAlpha(self) -> float: ...

def opaqueColor(c: object) -> bool: ...

class CMYKColor(Color):
    cyan: float
    magenta: float
    yellow: float
    black: float
    spotName: str | None
    density: float
    knockout: bool | None
    alpha: float
    def __init__(
        self,
        cyan: float = 0,
        magenta: float = 0,
        yellow: float = 0,
        black: float = 0,
        spotName: str | None = None,
        density: float = 1,
        knockout: bool | None = None,
        alpha: float = 1,
    ) -> None: ...
    def fader(self, n: int, reverse: bool = False) -> list[Self]: ...
    def cmyk(self) -> tuple[float, float, float, float]: ...
    def cmyka(self) -> tuple[float, float, float, float, float]: ...

class PCMYKColor(CMYKColor):
    def __init__(
        self,
        cyan: float,
        magenta: float,
        yellow: float,
        black: float,
        density: float = 100,
        spotName: str | None = None,
        knockout: bool | None = None,
        alpha: float = 100,
    ) -> None: ...

class CMYKColorSep(CMYKColor):
    def __init__(
        self,
        cyan: float = 0,
        magenta: float = 0,
        yellow: float = 0,
        black: float = 0,
        spotName: str | None = None,
        density: float = 1,
        alpha: float = 1,
    ) -> None: ...

class PCMYKColorSep(PCMYKColor, CMYKColorSep):
    def __init__(
        self,
        cyan: float = 0,
        magenta: float = 0,
        yellow: float = 0,
        black: float = 0,
        spotName: str | None = None,
        density: float = 100,
        alpha: float = 100,
    ) -> None: ...

def cmyk2rgb(cmyk: tuple[float, float, float, float], density: float = 1) -> tuple[float, float, float]: ...
def rgb2cmyk(r: float, g: float, b: float) -> tuple[float, float, float, float]: ...
def color2bw(colorRGB: Color) -> Color: ...
def HexColor(val: str | int, htmlOnly: bool = False, hasAlpha: bool = False) -> Color: ...
def linearlyInterpolatedColor(c0: _ColorT, c1: _ColorT, x0: float, x1: float, x: float) -> _ColorT: ...
@overload
def obj_R_G_B(
    c: Color | list[float] | tuple[float, float, float, float] | tuple[float, float, float],
) -> tuple[float, float, float]: ...
@overload
def obj_R_G_B(c: None) -> None: ...

transparent: Color
ReportLabBlueOLD: Color
ReportLabBlue: Color
ReportLabBluePCMYK: Color
ReportLabLightBlue: Color
ReportLabFidBlue: Color
ReportLabFidRed: Color
ReportLabGreen: Color
ReportLabLightGreen: Color
aliceblue: Color
antiquewhite: Color
aqua: Color
aquamarine: Color
azure: Color
beige: Color
bisque: Color
black: Color
blanchedalmond: Color
blue: Color
blueviolet: Color
brown: Color
burlywood: Color
cadetblue: Color
chartreuse: Color
chocolate: Color
coral: Color
cornflowerblue: Color
cornflower: Color
cornsilk: Color
crimson: Color
cyan: Color
darkblue: Color
darkcyan: Color
darkgoldenrod: Color
darkgray: Color
darkgrey: Color
darkgreen: Color
darkkhaki: Color
darkmagenta: Color
darkolivegreen: Color
darkorange: Color
darkorchid: Color
darkred: Color
darksalmon: Color
darkseagreen: Color
darkslateblue: Color
darkslategray: Color
darkslategrey: Color
darkturquoise: Color
darkviolet: Color
deeppink: Color
deepskyblue: Color
dimgray: Color
dimgrey: Color
dodgerblue: Color
firebrick: Color
floralwhite: Color
forestgreen: Color
fuchsia: Color
gainsboro: Color
ghostwhite: Color
gold: Color
goldenrod: Color
gray: Color
grey: Color
green: Color
greenyellow: Color
honeydew: Color
hotpink: Color
indianred: Color
indigo: Color
ivory: Color
khaki: Color
lavender: Color
lavenderblush: Color
lawngreen: Color
lemonchiffon: Color
lightblue: Color
lightcoral: Color
lightcyan: Color
lightgoldenrodyellow: Color
lightgreen: Color
lightgrey: Color
lightpink: Color
lightsalmon: Color
lightseagreen: Color
lightskyblue: Color
lightslategray: Color
lightslategrey: Color
lightsteelblue: Color
lightyellow: Color
lime: Color
limegreen: Color
linen: Color
magenta: Color
maroon: Color
mediumaquamarine: Color
mediumblue: Color
mediumorchid: Color
mediumpurple: Color
mediumseagreen: Color
mediumslateblue: Color
mediumspringgreen: Color
mediumturquoise: Color
mediumvioletred: Color
midnightblue: Color
mintcream: Color
mistyrose: Color
moccasin: Color
navajowhite: Color
navy: Color
oldlace: Color
olive: Color
olivedrab: Color
orange: Color
orangered: Color
orchid: Color
palegoldenrod: Color
palegreen: Color
paleturquoise: Color
palevioletred: Color
papayawhip: Color
peachpuff: Color
peru: Color
pink: Color
plum: Color
powderblue: Color
purple: Color
red: Color
rosybrown: Color
royalblue: Color
saddlebrown: Color
salmon: Color
sandybrown: Color
seagreen: Color
seashell: Color
sienna: Color
silver: Color
skyblue: Color
slateblue: Color
slategray: Color
slategrey: Color
snow: Color
springgreen: Color
steelblue: Color
tan: Color
teal: Color
thistle: Color
tomato: Color
turquoise: Color
violet: Color
wheat: Color
white: Color
whitesmoke: Color
yellow: Color
yellowgreen: Color
fidblue: Color
fidred: Color
fidlightblue: Color
ColorType: type[Color]

def colorDistance(col1: Color, col2: Color) -> float: ...
def cmykDistance(col1: Color, col2: Color) -> float: ...
def getAllNamedColors() -> dict[str, Color]: ...
@overload
def describe(aColor: Color, mode: Literal[0] = 0) -> None: ...
@overload
def describe(aColor: Color, mode: Literal[1]) -> str: ...
@overload
def describe(aColor: Color, mode: Literal[2]) -> tuple[str, float]: ...
def hue2rgb(m1: float, m2: float, h: float) -> float: ...
def hsl2rgb(h: float, s: float, l: float) -> tuple[float, float, float]: ...
@type_check_only
class _cssParse:
    def pcVal(self, v: str) -> float: ...
    def rgbPcVal(self, v: str) -> float: ...
    def rgbVal(self, v: str) -> float: ...
    def hueVal(self, v: str) -> float: ...
    def alphaVal(self, v: str, c: float = 1, n: str = "alpha") -> float: ...
    s: str
    def __call__(self, s: str) -> Color: ...

cssParse: _cssParse

@type_check_only
class _toColor:
    extraColorsNS: dict[str, Color]
    def __init__(self) -> None: ...
    def setExtraColorsNameSpace(self, NS: dict[str, Color]) -> None: ...
    def __call__(self, arg: _ConvertibleToColor, default: Color | None = None) -> Color: ...

toColor: _toColor

@overload
def toColorOrNone(arg: None, default: Color | None) -> None: ...
@overload
def toColorOrNone(arg: _ConvertibleToColor, default: Color | None = None) -> Color: ...
def setColors(**kw: _ConvertibleToColor) -> None: ...
def Whiter(c: _ColorT, f: float) -> _ColorT: ...
def Blacker(c: _ColorT, f: float) -> _ColorT: ...
def fade(aSpotColor: CMYKColor, percentages: Iterable[float]) -> list[CMYKColor]: ...



from _typeshed import Incomplete

from antlr4.Token import CommonToken as CommonToken

class TokenTagToken(CommonToken):
    __slots__ = ("tokenName", "label")
    tokenName: Incomplete
    label: Incomplete
    def __init__(self, tokenName: str, type: int, label: str | None = None) -> None: ...
    def getText(self): ...



from typing import Any, ClassVar
from typing_extensions import Self

import passlib.utils.handlers as uh

class _BcryptCommon(uh.SubclassBackendMixin, uh.TruncateMixin, uh.HasManyIdents, uh.HasRounds, uh.HasSalt, uh.GenericHandler):  # type: ignore[misc]
    name: ClassVar[str]
    checksum_size: ClassVar[int]
    checksum_chars: ClassVar[str]
    default_ident: ClassVar[str]
    ident_values: ClassVar[tuple[str, ...]]
    ident_aliases: ClassVar[dict[str, str]]
    min_salt_size: ClassVar[int]
    max_salt_size: ClassVar[int]
    salt_chars: ClassVar[str]
    final_salt_chars: ClassVar[str]
    default_rounds: ClassVar[int]
    min_rounds: ClassVar[int]
    max_rounds: ClassVar[int]
    rounds_cost: ClassVar[str]
    truncate_size: ClassVar[int | None]
    @classmethod
    def from_string(cls, hash) -> Self: ...  # type: ignore[override]
    @classmethod
    def needs_update(cls, hash, **kwds): ...  # type: ignore[override]
    @classmethod
    def normhash(cls, hash): ...

class _NoBackend(_BcryptCommon): ...
class _BcryptBackend(_BcryptCommon): ...
class _BcryptorBackend(_BcryptCommon): ...
class _PyBcryptBackend(_BcryptCommon): ...
class _OsCryptBackend(_BcryptCommon): ...
class _BuiltinBackend(_BcryptCommon): ...

class bcrypt(_NoBackend, _BcryptCommon):  # type: ignore[misc]
    backends: ClassVar[tuple[str, ...]]

class _wrapped_bcrypt(bcrypt):
    truncate_size: ClassVar[None]

class bcrypt_sha256(_wrapped_bcrypt):
    name: ClassVar[str]
    ident_values: ClassVar[tuple[str, ...]]
    ident_aliases: ClassVar[dict[str, str]]
    default_ident: ClassVar[str]
    version: ClassVar[int]
    @classmethod
    def using(cls, version=None, **kwds): ...  # type: ignore[override]
    prefix: Any
    @classmethod
    def identify(cls, hash): ...
    @classmethod
    def from_string(cls, hash) -> Self: ...  # type: ignore[override]
    def __init__(self, version=None, **kwds) -> None: ...

__all__ = ["bcrypt"]



from typing import Final

from .converters import BaseConverter, Converter, GenConverter, UnstructureStrategy
from .errors import (
    AttributeValidationNote,
    BaseValidationError,
    ClassValidationError,
    ForbiddenExtraKeysError,
    IterableValidationError,
    IterableValidationNote,
    StructureHandlerNotFoundError,
)
from .gen import override
from .v import transform_error

__all__ = [
    "structure",
    "unstructure",
    "get_structure_hook",
    "get_unstructure_hook",
    "register_structure_hook_func",
    "register_structure_hook",
    "register_unstructure_hook_func",
    "register_unstructure_hook",
    "structure_attrs_fromdict",
    "structure_attrs_fromtuple",
    "global_converter",
    "BaseConverter",
    "Converter",
    "AttributeValidationNote",
    "BaseValidationError",
    "ClassValidationError",
    "ForbiddenExtraKeysError",
    "GenConverter",
    "IterableValidationError",
    "IterableValidationNote",
    "override",
    "StructureHandlerNotFoundError",
    "transform_error",
    "UnstructureStrategy",
]

#: The global converter. Prefer creating your own if customizations are required.
global_converter: Final = Converter()

unstructure = global_converter.unstructure
structure = global_converter.structure
structure_attrs_fromtuple = global_converter.structure_attrs_fromtuple
structure_attrs_fromdict = global_converter.structure_attrs_fromdict
register_structure_hook = global_converter.register_structure_hook
register_structure_hook_func = global_converter.register_structure_hook_func
register_unstructure_hook = global_converter.register_unstructure_hook
register_unstructure_hook_func = global_converter.register_unstructure_hook_func
get_structure_hook: Final = global_converter.get_structure_hook
get_unstructure_hook: Final = global_converter.get_unstructure_hook



from collections.abc import Collection, Hashable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from numpy.typing import DTypeLike
from scipy.sparse import csc_array, csr_array  # type: ignore[import-untyped]  # pyright: ignore[reportMissingImports]

__all__ = ["incidence_matrix", "adjacency_matrix"]

@_dispatchable
def incidence_matrix(
    G: Graph[_Node],
    nodelist: Collection[_Node] | None = None,
    edgelist: (
        Collection[
            # Requiring tuples to represent an edge might be too strict as runtime does not check the type of
            # the collection. We can replace the tuples by `Collection[_Node | Hashable]` if people complain.
            tuple[_Node, _Node]  # for normal graphs, this is (u, v)
            | tuple[_Node, _Node, Hashable]  # for multigraphs, this is (u, v, key)
        ]
        | None
    ) = None,
    oriented: bool = False,
    weight: str | None = None,
    *,
    dtype: DTypeLike | None = None,
) -> csc_array: ...
@_dispatchable
def adjacency_matrix(
    G: Graph[_Node], nodelist: Collection[_Node] | None = None, dtype: DTypeLike | None = None, weight: str | None = "weight"
) -> csr_array: ...



from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: str

class Azure(VaultApiBase):
    def configure(
        self, subscription_id, tenant_id, client_id=None, client_secret=None, environment=None, mount_point="azure"
    ): ...
    def read_config(self, mount_point="azure"): ...
    def delete_config(self, mount_point="azure"): ...
    def create_or_update_role(self, name, azure_roles, ttl=None, max_ttl=None, mount_point="azure"): ...
    def list_roles(self, mount_point="azure"): ...
    def generate_credentials(self, name, mount_point="azure"): ...



from reportlab.graphics.shapes import Drawing, _DrawingEditorMixin
from reportlab.graphics.widgetbase import Widget
from reportlab.lib.attrmap import *
from reportlab.lib.validators import *

class AdjustableArrow(Widget):
    def __init__(self, **kwds) -> None: ...
    def draw(self): ...

class AdjustableArrowDrawing(_DrawingEditorMixin, Drawing):
    def __init__(self, width: int = 100, height: int = 63, *args, **kw) -> None: ...



from collections.abc import Generator

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["generate_edgelist", "write_edgelist", "parse_edgelist", "read_edgelist"]

@_dispatchable
def write_edgelist(
    G: Graph[_Node], path, comments: str = "#", delimiter: str = " ", data: bool = True, encoding: str = "utf-8"
) -> None: ...
@_dispatchable
def generate_edgelist(G: Graph[_Node], delimiter: str = " ", data: bool = True) -> Generator[str]: ...
@_dispatchable
def parse_edgelist(
    lines,
    comments: str | None = "#",
    delimiter: str | None = None,
    create_using: Graph[_Node] | None = None,
    nodetype=None,
    data=True,
): ...
@_dispatchable
def read_edgelist(
    path,
    comments: str | None = "#",
    delimiter: str | None = None,
    create_using=None,
    nodetype=None,
    data=True,
    edgetype=None,
    encoding: str | None = "utf-8",
): ...



from typing import Any, ClassVar

from . import get_random_bytes as get_random_bytes
from .base import Key

# Enable when we can use stubs from installed dependencies:
# from cryptography.hazmat import backends
class CryptographyECKey(Key):
    SHA256: Any
    SHA384: Any
    SHA512: Any
    hash_alg: Any
    cryptography_backend: Any
    prepared_key: Any
    def __init__(self, key, algorithm, cryptography_backend=...) -> None: ...
    def sign(self, msg): ...
    def verify(self, msg, sig): ...
    def is_public(self): ...
    def public_key(self): ...
    def to_pem(self): ...
    def to_dict(self): ...

class CryptographyRSAKey(Key):
    SHA256: Any
    SHA384: Any
    SHA512: Any
    RSA1_5: Any
    RSA_OAEP: Any
    RSA_OAEP_256: Any
    hash_alg: Any
    padding: Any
    cryptography_backend: Any
    prepared_key: Any
    def __init__(self, key, algorithm, cryptography_backend=...) -> None: ...
    def sign(self, msg): ...
    def verify(self, msg, sig): ...
    def is_public(self): ...
    def public_key(self): ...
    def to_pem(self, pem_format: str = "PKCS8"): ...
    def to_dict(self): ...
    def wrap_key(self, key_data): ...
    def unwrap_key(self, wrapped_key): ...

class CryptographyAESKey(Key):
    KEY_128: Any
    KEY_192: Any
    KEY_256: Any
    KEY_384: Any
    KEY_512: Any
    AES_KW_ALGS: Any
    MODES: Any
    IV_BYTE_LENGTH_MODE_MAP: ClassVar[dict[str, int]]
    def __init__(self, key, algorithm) -> None: ...
    def to_dict(self): ...
    def encrypt(self, plain_text, aad=None): ...
    def decrypt(self, cipher_text, iv=None, aad=None, tag=None): ...
    def wrap_key(self, key_data): ...
    def unwrap_key(self, wrapped_key): ...

class CryptographyHMACKey(Key):
    ALG_MAP: Any
    prepared_key: Any
    def __init__(self, key, algorithm) -> None: ...
    def to_dict(self): ...
    def sign(self, msg): ...
    def verify(self, msg, sig): ...



from typing import Any

from passlib.utils import SequenceMixin

def lookup_hash(digest, return_unknown: bool = False, required: bool = True): ...
def norm_hash_name(name, format: str = "hashlib"): ...

class HashInfo(SequenceMixin):
    name: Any
    iana_name: Any
    aliases: Any
    const: Any
    digest_size: Any
    block_size: Any
    error_text: Any
    unknown: bool
    def __init__(self, const, names, required: bool = True) -> None: ...
    @property
    def supported(self): ...
    @property
    def supported_by_fastpbkdf2(self): ...
    @property
    def supported_by_hashlib_pbkdf2(self): ...

def compile_hmac(digest, key, multipart: bool = False): ...
def pbkdf1(digest, secret, salt, rounds, keylen=None): ...
def pbkdf2_hmac(digest, secret, salt, rounds, keylen=None): ...

__all__ = [
    # hash utils
    "lookup_hash",
    "HashInfo",
    "norm_hash_name",
    # hmac utils
    "compile_hmac",
    # kdfs
    "pbkdf1",
    "pbkdf2_hmac",
]



from typing import Final

from .html5parser import HTMLParser as HTMLParser, parse as parse, parseFragment as parseFragment
from .serializer import serialize as serialize
from .treebuilders import getTreeBuilder as getTreeBuilder
from .treewalkers import getTreeWalker as getTreeWalker

__all__ = ["HTMLParser", "parse", "parseFragment", "getTreeBuilder", "getTreeWalker", "serialize"]

__version__: Final[str]



from _typeshed import Incomplete
from typing import ClassVar, Final

from docutils import nodes
from docutils.writers import _html_base

__docformat__: Final = "reStructuredText"

class Writer(_html_base.Writer):
    default_stylesheets: ClassVar[list[str]]
    default_stylesheet_dirs: ClassVar[list[str]]
    default_template: ClassVar[str]
    config_section: ClassVar[str]
    translator_class: type[HTMLTranslator]

class HTMLTranslator(_html_base.HTMLTranslator):
    content_type: ClassVar[str]
    content_type_mathml: ClassVar[str]
    object_image_types: ClassVar[dict[str, str]]
    def set_first_last(self, node) -> None: ...
    def visit_address(self, node: nodes.address) -> None: ...
    def depart_address(self, node: nodes.address) -> None: ...
    def visit_admonition(self, node: nodes.admonition) -> None: ...
    def depart_admonition(self, node: nodes.admonition | None = None) -> None: ...
    def visit_author(self, node: nodes.author) -> None: ...
    author_in_authors: bool
    def depart_author(self, node: nodes.author) -> None: ...
    def visit_authors(self, node: nodes.authors) -> None: ...
    def depart_authors(self, node: nodes.authors) -> None: ...
    def visit_colspec(self, node: nodes.colspec) -> None: ...
    def depart_colspec(self, node: nodes.colspec) -> None: ...
    def is_compactable(self, node: nodes.Element) -> bool: ...
    def visit_citation(self, node: nodes.citation) -> None: ...
    def depart_citation(self, node: nodes.citation) -> None: ...
    def visit_citation_reference(self, node: nodes.citation_reference) -> None: ...
    def depart_citation_reference(self, node: nodes.citation_reference) -> None: ...
    def visit_classifier(self, node: nodes.classifier) -> None: ...
    def depart_classifier(self, node: nodes.classifier) -> None: ...
    def visit_compound(self, node: nodes.compound) -> None: ...
    def depart_compound(self, node: nodes.compound) -> None: ...
    def visit_definition(self, node: nodes.definition) -> None: ...
    def depart_definition(self, node: nodes.definition) -> None: ...
    def visit_definition_list(self, node: nodes.definition_list) -> None: ...
    def depart_definition_list(self, node: nodes.definition_list) -> None: ...
    def visit_definition_list_item(self, node: nodes.definition_list_item) -> None: ...
    def depart_definition_list_item(self, node: nodes.definition_list_item) -> None: ...
    def visit_description(self, node: nodes.description) -> None: ...
    def depart_description(self, node: nodes.description) -> None: ...
    in_docinfo: bool
    def visit_docinfo(self, node: nodes.docinfo) -> None: ...
    docinfo: Incomplete
    body: Incomplete
    def depart_docinfo(self, node: nodes.docinfo) -> None: ...
    def visit_docinfo_item(self, node, name, meta: bool = True) -> None: ...
    def depart_docinfo_item(self) -> None: ...
    def visit_doctest_block(self, node) -> None: ...
    def depart_doctest_block(self, node) -> None: ...
    def visit_entry(self, node) -> None: ...
    def depart_entry(self, node) -> None: ...
    compact_p: Incomplete
    compact_simple: Incomplete
    def visit_enumerated_list(self, node) -> None: ...
    def depart_enumerated_list(self, node) -> None: ...
    def visit_field(self, node) -> None: ...
    def depart_field(self, node) -> None: ...
    def visit_field_body(self, node) -> None: ...
    def depart_field_body(self, node) -> None: ...
    compact_field_list: bool
    def visit_field_list(self, node: nodes.field_list) -> None: ...
    def depart_field_list(self, node: nodes.field_list) -> None: ...
    def visit_field_name(self, node: nodes.field_name) -> None: ...
    def depart_field_name(self, node: nodes.field_name) -> None: ...
    def visit_footnote(self, node: nodes.footnote) -> None: ...
    def footnote_backrefs(self, node: nodes.footnote) -> None: ...
    def depart_footnote(self, node: nodes.footnote) -> None: ...
    def visit_footnote_reference(self, node: nodes.footnote_reference) -> None: ...
    def depart_footnote_reference(self, node: nodes.footnote_reference) -> None: ...
    def visit_generated(self, node: nodes.generated) -> None: ...
    def visit_image(self, node: nodes.image) -> None: ...
    def depart_image(self, node: nodes.image) -> None: ...
    def visit_label(self, node: nodes.label) -> None: ...
    def depart_label(self, node: nodes.label) -> None: ...
    def visit_list_item(self, node: nodes.list_item) -> None: ...
    def depart_list_item(self, node: nodes.list_item) -> None: ...
    def visit_literal(self, node: nodes.literal) -> None: ...
    def depart_literal(self, node: nodes.literal) -> None: ...
    def visit_literal_block(self, node: nodes.literal_block) -> None: ...
    def depart_literal_block(self, node: nodes.literal_block) -> None: ...
    def visit_option_group(self, node: nodes.option_group) -> None: ...
    def depart_option_group(self, node: nodes.option_group) -> None: ...
    def visit_option_list(self, node: nodes.option_list) -> None: ...
    def depart_option_list(self, node: nodes.option_list) -> None: ...
    def visit_option_list_item(self, node: nodes.option_list_item) -> None: ...
    def depart_option_list_item(self, node: nodes.option_list_item) -> None: ...
    def should_be_compact_paragraph(self, node: nodes.Element) -> bool: ...
    def visit_paragraph(self, node: nodes.paragraph) -> None: ...
    def depart_paragraph(self, node: nodes.paragraph) -> None: ...
    in_sidebar: bool
    def visit_sidebar(self, node: nodes.sidebar) -> None: ...
    def depart_sidebar(self, node: nodes.sidebar) -> None: ...
    def visit_subscript(self, node: nodes.subscript) -> None: ...
    def depart_subscript(self, node: nodes.subscript) -> None: ...
    in_document_title: int
    def visit_subtitle(self, node: nodes.subtitle) -> None: ...
    subtitle: list[Incomplete]
    def depart_subtitle(self, node: nodes.subtitle) -> None: ...
    def visit_superscript(self, node: nodes.superscript) -> None: ...
    def depart_superscript(self, node: nodes.superscript) -> None: ...
    def visit_system_message(self, node: nodes.system_message) -> None: ...
    def depart_system_message(self, node: nodes.system_message) -> None: ...
    def visit_table(self, node: nodes.table) -> None: ...
    def depart_table(self, node: nodes.table) -> None: ...
    def visit_tbody(self, node: nodes.tbody) -> None: ...
    def depart_tbody(self, node: nodes.tbody) -> None: ...
    def visit_term(self, node: nodes.term) -> None: ...
    def depart_term(self, node: nodes.term) -> None: ...
    def visit_thead(self, node: nodes.thead) -> None: ...
    def depart_thead(self, node: nodes.thead) -> None: ...
    def section_title_tags(self, node: nodes.Element) -> tuple[str, str]: ...

class SimpleListChecker(_html_base.SimpleListChecker):
    def visit_list_item(self, node: nodes.list_item) -> None: ...
    def visit_paragraph(self, node: nodes.paragraph) -> None: ...
    def visit_definition_list(self, node: nodes.definition_list) -> None: ...
    def visit_docinfo(self, node: nodes.docinfo) -> None: ...



from __future__ import annotations

from typing import Any

import grpc


@grpc.Call.register
class CallProxy:
    def __init__(self, target: grpc.Call) -> None:
        self._target = target

    def __getattr__(self, name: str) -> Any:
        return getattr(self._target, name)



import sys
from _typeshed import SupportsWrite
from asyncio import Future
from dataclasses import dataclass
from types import FrameType
from typing import Any, overload

if sys.version_info >= (3, 14):
    __all__ = ("capture_call_graph", "format_call_graph", "print_call_graph", "FrameCallGraphEntry", "FutureCallGraph")

    @dataclass(frozen=True, slots=True)
    class FrameCallGraphEntry:
        frame: FrameType

    @dataclass(frozen=True, slots=True)
    class FutureCallGraph:
        future: Future[Any]
        call_stack: tuple[FrameCallGraphEntry, ...]
        awaited_by: tuple[FutureCallGraph, ...]

    @overload
    def capture_call_graph(future: None = None, /, *, depth: int = 1, limit: int | None = None) -> FutureCallGraph | None: ...
    @overload
    def capture_call_graph(future: Future[Any], /, *, depth: int = 1, limit: int | None = None) -> FutureCallGraph | None: ...
    def format_call_graph(future: Future[Any] | None = None, /, *, depth: int = 1, limit: int | None = None) -> str: ...
    def print_call_graph(
        future: Future[Any] | None = None, /, *, file: SupportsWrite[str] | None = None, depth: int = 1, limit: int | None = None
    ) -> None: ...



import datetime
from typing import ClassVar

from .core import UnitedStates

class Georgia(UnitedStates):
    label_washington_birthday_december: ClassVar[str]
    def get_washington_birthday_december(self, year: int) -> tuple[datetime.date, str]: ...
    def get_robert_lee_birthday(self, year: int) -> tuple[datetime.date, str]: ...



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from numpy.random import RandomState

__all__ = ["diameter"]

@_dispatchable
def diameter(G: Graph[_Node], seed: int | RandomState | None = None): ...



from _typeshed import Unused
from collections.abc import Callable, Iterable, Sequence
from typing import Any, Final, Literal, TypedDict, TypeVar, overload, type_check_only
from typing_extensions import TypeAlias, Unpack

from reportlab.lib.styles import ParagraphStyle, PropertySet
from reportlab.pdfgen.canvas import Canvas
from reportlab.platypus.doctemplate import IndexingFlowable, _CanvasMaker
from reportlab.platypus.tables import TableStyle

_T = TypeVar("_T")
_Entry: TypeAlias = tuple[int, str, int] | tuple[int, str, int, str | None] | Sequence[int | str | None]
_SequencerFormat: TypeAlias = Literal["I", "i", "123", "ABC", "abc"]

@type_check_only
class _TableOfContentsKwargs(TypedDict, total=False):
    rightColumnWidth: float
    levelStyles: list[PropertySet]  # should be ParagraphStyle
    tableStyle: TableStyle
    dotsMinLevel: int
    formatter: Callable[[int], str] | None

@type_check_only
class _SimpleIndexKwargs(TypedDict, total=False):
    style: Iterable[PropertySet] | PropertySet | None  # should be ParagraphStyle
    dot: str | None
    tableStyle: TableStyle | None
    headers: bool
    name: str | None
    format: _SequencerFormat
    offset: int

__version__: Final[str]

def unquote(txt: str) -> str: ...
def drawPageNumbers(
    canvas: Canvas,
    style: PropertySet,  # should be ParagraphStyle
    pages: Iterable[tuple[int | str, Unused]],
    availWidth: float,
    availHeight: float,
    dot: str = " . ",
    formatter: Unused | None = None,
) -> None: ...

delta: float
epsilon: float
defaultLevelStyles: list[ParagraphStyle]
defaultTableStyle: TableStyle

class TableOfContents(IndexingFlowable):
    rightColumnWidth: float
    levelStyles: list[ParagraphStyle]
    tableStyle: TableStyle
    dotsMinLevel: int
    formatter: Callable[[int], str] | None
    def __init__(self, **kwds: Unpack[_TableOfContentsKwargs]) -> None: ...
    def isIndexing(self) -> Literal[1]: ...
    def isSatisfied(self) -> bool: ...
    def clearEntries(self) -> None: ...
    def getLevelStyle(self, n: int) -> ParagraphStyle: ...
    def addEntry(self, level: int, text: str, pageNum: int, key: str | None = None) -> None: ...
    def addEntries(self, listOfEntries: Iterable[_Entry]) -> None: ...

@overload
def makeTuple(x: tuple[_T, ...]) -> tuple[_T, ...]: ...
@overload
def makeTuple(x: list[_T]) -> tuple[_T, ...]: ...
@overload
def makeTuple(x: _T) -> tuple[_T, ...]: ...

class SimpleIndex(IndexingFlowable):
    # NOTE: Will be a list after getLevelStyle is called
    textStyle: ParagraphStyle | Iterable[ParagraphStyle] | list[ParagraphStyle]
    tableStyle: TableStyle
    dot: str | None
    headers: bool
    name: str
    formatFunc: Callable[[int], str]
    offset: float
    def __init__(self, **kwargs: Unpack[_SimpleIndexKwargs]) -> None: ...
    def getFormatFunc(self, formatName): ...
    def setup(
        self,
        style: PropertySet | None = None,  # should be ParagraphStyle
        dot: str | None = None,
        tableStyle: TableStyle | None = None,
        headers: bool = True,
        name: str | None = None,
        format: _SequencerFormat = "123",
        offset: float = 0,
    ) -> None: ...
    def __call__(self, canv: Canvas, kind: str | None, label: str) -> None: ...
    def getCanvasMaker(self, canvasmaker: _CanvasMaker = ...) -> _CanvasMaker: ...
    def isIndexing(self) -> Literal[1]: ...
    def isSatisfied(self) -> bool: ...
    def clearEntries(self) -> None: ...
    def addEntry(self, text: str, pageNum: tuple[int, str], key: str | None = None) -> None: ...
    def draw(self) -> None: ...
    def getLevelStyle(self, n: int) -> ParagraphStyle: ...

AlphabeticIndex = SimpleIndex

def listdiff(l1: list[Any], l2: list[_T]) -> tuple[int, list[_T]]: ...

class ReferenceText(IndexingFlowable):
    textPattern: str
    target: str
    paraStyle: ParagraphStyle
    def __init__(self, textPattern: str, targetKey: str) -> None: ...



import re
from _typeshed import Unused
from collections.abc import Callable, Collection, Iterable, Iterator, Mapping, Sequence
from functools import cached_property
from json.decoder import JSONDecoder
from typing import Any, ClassVar, Literal, TypedDict, TypeVar, overload, type_check_only
from typing_extensions import Never, Required, TypeAlias, deprecated
from urllib.request import Request, _DataType
from xml.etree import ElementTree as ET

from ..cache import Cache
from ..cookies import LenientSimpleCookie, YoutubeDLCookieJar
from ..networking.common import Response, _RequestData
from ..networking.impersonate import ImpersonateTarget
from ..utils._utils import NO_DEFAULT, ExtractorError, FormatSorter, RetryManager as _RetryManager, classproperty
from ..YoutubeDL import YoutubeDL

@type_check_only
class _InfoDict(TypedDict, total=False):
    age_limit: int
    availability: Literal["private", "premium_only", "subscriber_only", "needs_auth", "unlisted", "public"] | None
    available_at: int
    creator: str | None
    comment_count: int | None
    duration: int | None
    formats: list[dict[str, Any]] | None
    id: Required[str]
    like_count: int | None
    tags: list[str] | None
    thumbnail: str | None
    timestamp: int | float | None
    title: str | None
    uploader: str | None
    url: str | None

_StrNoDefaultOrNone: TypeAlias = str | None | type[NO_DEFAULT]
_T = TypeVar("_T")

class InfoExtractor:
    IE_DESC: ClassVar[str | bool | None]
    SEARCH_KEY: ClassVar[str | None]
    def _login_hint(self, method: _StrNoDefaultOrNone = ..., netrc: str | None = None) -> dict[str, str]: ...
    def __init__(self, downloader: YoutubeDL | None = None) -> None: ...
    @classmethod
    def _match_valid_url(cls, url: str) -> re.Match[str] | None: ...
    @classmethod
    def suitable(cls, url: str) -> re.Match[str] | None: ...
    @classmethod
    def get_temp_id(cls, url: str) -> str | None: ...
    @classmethod
    def working(cls) -> bool: ...
    @classmethod
    def supports_login(cls) -> bool: ...
    def initialize(self) -> None: ...
    def extract(self, url: str) -> Iterator[_InfoDict]: ...
    def set_downloader(self, downloader: YoutubeDL) -> None: ...
    @property
    def cache(self) -> Cache: ...
    @property
    def cookiejar(self) -> YoutubeDLCookieJar: ...
    def _initialize_pre_login(self) -> None: ...
    def _perform_login(self, username: str, password: str) -> None: ...
    def _real_initialize(self) -> None: ...
    @classmethod
    def ie_key(cls) -> str: ...
    @property
    def IE_NAME(cls) -> str: ...
    def _create_request(
        self,
        url_or_request: str | Request,
        data: _DataType | None = None,
        headers: Mapping[str, str] | None = None,
        query: str | Mapping[str, str] | None = None,
        extensions: Mapping[str, Any] | None = None,
    ) -> Request: ...
    def _download_webpage_handle(
        self,
        url_or_request: str | Request,
        video_id: str,
        note: str | None = None,
        errnote: str | None = None,
        fatal: bool = True,
        encoding: str | None = None,
        data: _DataType | None = None,
        headers: Mapping[str, str] | None = {},
        query: str | Mapping[str, str] | None = {},
        expected_status: int | None = None,
        impersonate: ImpersonateTarget | str | bool | Collection[str | ImpersonateTarget] | None = None,
        require_impersonation: bool = False,
    ) -> tuple[str, Response] | Literal[False]: ...
    @staticmethod
    def _guess_encoding_from_content(content_type: str, webpage_bytes: bytes) -> str: ...
    def _webpage_read_content(
        self,
        urlh: Response,
        url_or_request: str | Request,
        video_id: str,
        note: str | None = None,
        errnote: str | None = None,
        fatal: bool = True,
        prefix: str | None = None,
        encoding: str | None = None,
        data: _RequestData | None = None,
    ) -> str | Literal[False]: ...
    def _parse_json(
        self,
        json_string: str,
        video_id: str,
        transform_source: Callable[..., str] | None = None,
        fatal: bool = True,
        errnote: str | None = None,
        *,
        cls: type[JSONDecoder] | None = None,
        object_hook: Callable[[dict[Any, Any]], Any] | None = None,
        parse_float: Callable[[str], Any] | None = None,
        parse_int: Callable[[str], Any] | None = None,
        parse_constant: Callable[[str], Any] | None = None,
        object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = None,
    ) -> Any: ...
    def _parse_socket_response_as_json(
        self,
        data: str,
        video_id: str,
        cls: type[JSONDecoder] | None = None,
        object_hook: Callable[[dict[Any, Any]], Any] | None = None,
        parse_float: Callable[[str], Any] | None = None,
        parse_int: Callable[[str], Any] | None = None,
        parse_constant: Callable[[str], Any] | None = None,
        object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = None,
    ) -> Any: ...
    # *args and **kwargs are passed to self._downloader.report_warning().
    def report_warning(
        self, msg: str, video_id: str | None = None, *args: Any, only_once: bool = False, **kwargs: Any
    ) -> None: ...
    def to_screen(
        self, msg: str, message: str, skip_eol: bool = False, quiet: bool | None = None, only_once: bool = False
    ) -> None: ...
    def write_debug(self, msg: str, only_once: bool = False) -> None: ...
    # *args and **kwargs are passed to .params.get() where params is normally a mapping but is not required to be.
    def get_param(self, name: str, default: Any = None, *args: Any, **kwargs: Any) -> Any: ...
    @overload
    def report_drm(self, video_id: str, partial: type[NO_DEFAULT] = ...) -> None: ...
    @overload
    @deprecated("InfoExtractor.report_drm no longer accepts the argument partial")
    def report_drm(self, video_id: str, partial: bool) -> None: ...
    def report_extraction(self, id_or_name: str) -> None: ...
    def report_download_webpage(self, video_id: str) -> None: ...
    def report_age_confirmation(self) -> None: ...
    def report_login(self) -> None: ...
    def raise_login_required(
        self,
        msg: str = "This video is only available for registered users",
        metadata_available: bool = False,
        method: str | type[NO_DEFAULT] = ...,
    ) -> None: ...
    def raise_geo_restricted(
        self, msg: str = ..., countries: Collection[str] | None = None, metadata_available: bool = False
    ) -> None: ...
    @overload
    def raise_no_formats(
        self, msg: str | ExtractorError, expected: Literal[False] = False, video_id: str | None = None
    ) -> Never: ...
    @overload
    def raise_no_formats(self, msg: str | ExtractorError, expected: Literal[True], video_id: str | None = None) -> None: ...
    @staticmethod
    def url_result(
        url: str,
        ie: InfoExtractor | None = None,
        video_id: str | None = None,
        video_title: str | None = None,
        *,
        url_transparent: bool = False,
        **kwargs: Any,  # Added to the dict return value.
    ) -> dict[str, Any]: ...
    @classmethod
    def playlist_from_matches(
        cls,
        matches: Sequence[str],
        playlist_id: str | None = None,
        playlist_title: str | None = None,
        getter: Callable[..., Any] = ...,
        ie: InfoExtractor | None = None,
        video_kwargs: Mapping[str, Any] | None = None,
        **kwargs: Any,  # Added to the dict return value.
    ) -> dict[str, Any]: ...
    @staticmethod
    def playlist_result(
        entries: Iterable[_InfoDict],
        playlist_id: str | None = None,
        playlist_title: str | None = None,
        playlist_description: str | None = None,
        *,
        multi_video: bool = False,
        **kwargs: Any,  # Added to the dict return value.
    ) -> _InfoDict: ...
    def http_scheme(self) -> str: ...
    @classmethod
    def get_testcases(cls, include_onlymatching: bool = False) -> Iterator[dict[str, Any]]: ...
    @classmethod
    def get_webpage_testcases(cls) -> Iterator[dict[str, Any]]: ...
    @property
    def age_limit(cls) -> int: ...
    @classmethod
    def is_single_video(cls, url: str) -> bool: ...
    @classmethod
    def is_suitable(cls, age_limit: int) -> bool: ...
    @classmethod
    def description(cls, *, markdown: bool = True, search_examples: Sequence[str] | None = None) -> str: ...
    # Calls _get_subtitles which only raises NotImplementedError here.
    def extract_subtitles(self, *args: Any, **kwargs: Any) -> list[dict[str, Any]]: ...
    def _configuration_arg(self, key: str, default: Any = ..., *, ie_key: str | None = None, casesense: bool = False) -> Any: ...
    # These are dynamically created.
    def _download_xml_handle(
        self,
        url_or_request: str | Request,
        video_id: str,
        note: str | None = "Downloading XML",
        errnote: str | None = "Unable to download XML",
        transform_source: Callable[..., str] | None = None,
        fatal: bool = True,
        encoding: str | None = None,
        data: _DataType | None = None,
        headers: Mapping[str, str] = {},
        query: Mapping[str, str] = {},
        expected_status: int | None = None,
        impersonate: ImpersonateTarget | str | bool | Collection[str | ImpersonateTarget] | None = None,
        require_impersonation: bool = False,
    ) -> tuple[ET.ElementTree, Response]: ...
    def _download_xml(
        self,
        url_or_request: str | Request,
        video_id: str,
        note: str | None = "Downloading XML",
        errnote: str | None = "Unable to download XML",
        transform_source: Callable[..., str] | None = None,
        fatal: bool = True,
        encoding: str | None = None,
        data: _DataType | None = None,
        headers: Mapping[str, str] = {},
        query: Mapping[str, str] = {},
        expected_status: int | None = None,
        impersonate: ImpersonateTarget | str | bool | Collection[str | ImpersonateTarget] | None = None,
        require_impersonation: bool = False,
    ) -> ET.ElementTree: ...
    def _download_socket_json_handle(
        self,
        url_or_request: str | Request,
        video_id: str,
        note: str | None = "Polling socket",
        errnote: str | None = "Unable to poll socket",
        transform_source: Callable[..., str] | None = None,
        fatal: bool = True,
        encoding: str | None = None,
        data: _DataType | None = None,
        headers: Mapping[str, str] = {},
        query: Mapping[str, str] = {},
        expected_status: int | None = None,
        impersonate: ImpersonateTarget | str | bool | Collection[str | ImpersonateTarget] | None = None,
        require_impersonation: bool = False,
    ) -> tuple[dict[str, Any], Response]: ...
    def _download_socket_json(
        self,
        url_or_request: str | Request,
        video_id: str,
        note: str | None = "Polling socket",
        errnote: str | None = "Unable to poll socket",
        transform_source: Callable[..., str] | None = None,
        fatal: bool = True,
        encoding: str | None = None,
        data: _DataType | None = None,
        headers: Mapping[str, str] = {},
        query: Mapping[str, str] = {},
        expected_status: int | None = None,
        impersonate: ImpersonateTarget | str | bool | Collection[str | ImpersonateTarget] | None = None,
        require_impersonation: bool = False,
    ) -> dict[str, Any]: ...
    def _download_json_handle(
        self,
        url_or_request: str | Request,
        video_id: str,
        note: str | None = "Downloading JSON metadata",
        errnote: str | None = "Unable to download JSON metadata",
        transform_source: Callable[..., str] | None = None,
        fatal: bool = True,
        encoding: str | None = None,
        data: _DataType | None = None,
        headers: Mapping[str, str] = {},
        query: Mapping[str, str] = {},
        expected_status: int | None = None,
        impersonate: ImpersonateTarget | str | bool | Collection[str | ImpersonateTarget] | None = None,
        require_impersonation: bool = False,
    ) -> tuple[dict[str, Any], Response]: ...
    def _download_json(
        self,
        url_or_request: str | Request,
        video_id: str,
        note: str | None = "Downloading JSON metadata",
        errnote: str | None = "Unable to download JSON metadata",
        transform_source: Callable[..., str] | None = None,
        fatal: bool = True,
        encoding: str | None = None,
        data: _DataType | None = None,
        headers: Mapping[str, str] = {},
        query: Mapping[str, str] = {},
        expected_status: int | None = None,
        impersonate: ImpersonateTarget | str | bool | Collection[str | ImpersonateTarget] | None = None,
        require_impersonation: bool = False,
    ) -> dict[str, Any]: ...
    def _download_webpage(
        self,
        url_or_request: str | Request,
        video_id: str,
        note: str | None = None,
        errnote: str | None = None,
        fatal: bool = True,
        tries: int = 1,
        timeout: float | type[NO_DEFAULT] = ...,
        # Remaining arguments are collected with *args, **kwargs and
        # forwarded to _download_webpage_handle().
        encoding: str | None = ...,
        data: _DataType | None = ...,
        headers: Mapping[str, str] = ...,
        query: Mapping[str, str] = ...,
        expected_status: int | None = ...,
        impersonate: ImpersonateTarget | str | bool | Collection[str | ImpersonateTarget] | None = ...,
        require_impersonation: bool = ...,
    ) -> str: ...
    def _parse_xml(
        self,
        xml_string: str,
        video_id: str,
        transform_source: Callable[..., str] | None = None,
        fatal: bool = True,
        errnote: str | None = None,
    ) -> ET.Element: ...
    def _parse_mpd_formats(
        self, mpd_doc: ET.Element, mpd_id: str | None = ..., mpd_base_url: str = ..., mpd_url: str | None = ...
    ) -> list[Any]: ...
    def _real_extract(self, url: str) -> _InfoDict: ...
    @staticmethod
    def _availability(
        is_private: bool | None = None,
        needs_premium: bool | None = None,
        needs_subscription: bool | None = None,
        needs_auth: bool | None = None,
        is_unlisted: bool | None = None,
    ) -> Literal["needs_auth", "premium_only", "private", "public", "subscriber_only", "unlisted"] | None: ...
    def _request_webpage(
        self,
        url_or_request: str | Request,
        video_id: str,
        note: str | None = None,
        errnote: str | None = None,
        fatal: bool = True,
        data: _DataType | None = None,
        headers: Mapping[str, str] | None = None,
        query: Mapping[str, str] | None = None,
        expected_status: int | None = None,
        impersonate: ImpersonateTarget | str | bool | Collection[str | ImpersonateTarget] | None = None,
        require_impersonation: bool = False,
    ) -> Response | Literal[False]: ...
    @classmethod
    def _match_id(cls, url: str) -> str: ...
    def _search_regex(
        self,
        pattern: str | re.Pattern[str],
        string: str | None,
        name: str,
        default: _StrNoDefaultOrNone = ...,
        fatal: bool = True,
        flags: int = 0,
        group: tuple[int, ...] | list[int] | None = None,
    ) -> str: ...
    def _search_json(
        self,
        start_pattern: str | re.Pattern[str],
        string: str | None,
        name: str,
        video_id: str,
        *,
        end_pattern: str | re.Pattern[str] = "",
        contains_pattern: str | re.Pattern[str] = r"{(?s:.+)}",
        fatal: bool = True,
        default: _StrNoDefaultOrNone = ...,
        cls: type[JSONDecoder] | None = None,
        object_hook: Callable[[dict[Any, Any]], Any] | None = None,
        parse_float: Callable[[str], Any] | None = None,
        parse_int: Callable[[str], Any] | None = None,
        parse_constant: Callable[[str], Any] | None = None,
        object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = None,
    ) -> Any: ...
    def _html_search_regex(
        self,
        pattern: str | re.Pattern[str],
        string: str,
        name: str,
        default: _StrNoDefaultOrNone = ...,
        fatal: bool = True,
        flags: int = 0,
        group: int | None = None,
    ) -> str | tuple[str, ...]: ...
    def _get_netrc_login_info(self, netrc_machine: str | None = None) -> tuple[str | None, str | None]: ...
    def _get_login_info(
        self, username_option: str = "username", password_option: str = "password", netrc_machine: str | None = None
    ) -> tuple[str | None, str | None]: ...
    def _get_tfa_info(self, note: str = "two-factor verification code") -> str: ...
    @staticmethod
    def _og_regexes(prop: str) -> list[str]: ...
    @staticmethod
    def _meta_regex(prop: str) -> str: ...
    def _og_search_property(
        self, prop: str, html: str, name: str | None = None, *, default: type[NO_DEFAULT] | str = ..., fatal: bool = False
    ) -> str | None: ...
    def _og_search_thumbnail(self, html: str, *, default: type[NO_DEFAULT] | str = ...) -> str | None: ...
    def _og_search_description(self, html: str, *, default: type[NO_DEFAULT] | str = ...) -> str | None: ...
    def _og_search_title(self, html: str, *, fatal: bool = False, default: type[NO_DEFAULT] | str = ...) -> str | None: ...
    def _og_search_video_url(
        self,
        html: str,
        name: str = "video url",
        secure: bool = True,
        *,
        default: type[NO_DEFAULT] | str = ...,
        fatal: bool = False,
    ) -> str | None: ...
    def _og_search_url(self, html: str, *, default: type[NO_DEFAULT] | str = ..., fatal: bool = False) -> str | None: ...
    def _html_extract_title(
        self,
        html: str,
        name: str = "title",
        *,
        default: type[NO_DEFAULT] | str = ...,
        flags: int = 0,
        group: tuple[int, ...] | list[int] | None = None,
        fatal: bool = False,
    ) -> str | None: ...
    def _html_search_meta(self, name: str, html: str, display_name: str | None = None, fatal: bool = False) -> str | None: ...
    def _dc_search_uploader(self, html: str) -> str | None: ...
    @staticmethod
    def _rta_search(html: str) -> int: ...
    def _media_rating_search(self, html: str) -> int: ...
    def _family_friendly_search(self, html: str) -> int: ...
    def _twitter_search_player(self, html: str) -> str | None: ...
    def _yield_json_ld(
        self, html: str, video_id: str, *, fatal: bool = True, default: type[NO_DEFAULT] | bool = ...
    ) -> Iterator[dict[str, Any]]: ...
    def _search_json_ld(
        self,
        html: str,
        video_id: str,
        expected_type: Iterable[str] | str | None = None,
        *,
        fatal: bool = True,
        default: type[NO_DEFAULT] | bool = ...,
    ) -> dict[str, Any]: ...
    # json_ld parameter is passed to json.loads().
    def _json_ld(
        self, json_ld: Any, video_id: str, fatal: bool = True, expected_type: Iterable[str] | str | None = None
    ) -> dict[str, Any]: ...
    def _search_nextjs_data(
        self, webpage: str, video_id: str, *, fatal: bool = True, default: type[NO_DEFAULT] | bool = ..., **kw: Any
    ) -> Any: ...
    def _search_nuxt_data(
        self,
        webpage: str,
        video_id: str,
        context_name: str = "__NUXT__",
        *,
        fatal: bool = True,
        traverse: tuple[str, int] = ("data", 0),
    ) -> Any: ...
    @staticmethod
    def _hidden_inputs(html: str) -> dict[str, Any]: ...
    def _form_hidden_inputs(self, form_id: str, html: str) -> dict[str, Any]: ...
    @classproperty
    @deprecated(
        "yt_dlp.InfoExtractor.FormatSort is deprecated and may be removed in the future. Use yt_dlp.utils.FormatSorter instead"
    )
    def FormatSort(self) -> FormatSorter: ...
    def _check_formats(self, formats: list[dict[str, Any]], video_id: str) -> None: ...
    @staticmethod
    def _remove_duplicate_formats(formats: list[dict[str, Any]]) -> None: ...
    def _is_valid_url(self, url: str, video_id: str, item: str = "video", headers: Mapping[str, Any] = {}) -> bool: ...
    def _proto_relative_url(self, url: str, scheme: str | None = None) -> str: ...
    def _sleep(self, timeout: float, video_id: str, msg_template: str | None = None) -> None: ...
    def _extract_f4m_formats(
        self,
        manifest_url: str,
        video_id: str,
        preference: Any = None,
        quality: Any = None,
        f4m_id: str | None = None,
        transform_source: Callable[..., str] = ...,
        fatal: bool = True,
        m3u8_id: str | None = None,
        data: str | None = None,
        headers: Mapping[str, Any] = {},
        query: Mapping[str, Any] = {},
    ) -> list[dict[str, Any]]: ...
    def _parse_f4m_formats(
        self,
        manifest: str,
        manifest_url: str,
        video_id: str,
        preference: Any = None,
        quality: Any = None,
        f4m_id: str | None = None,
        transform_source: Callable[..., str] = ...,
        fatal: bool = True,
        m3u8_id: str | None = None,
    ) -> list[dict[str, Any]]: ...
    def _m3u8_meta_format(
        self, m3u8_url: str, ext: str | None = None, preference: Any = None, quality: Any = None, m3u8_id: str | None = None
    ) -> dict[str, Any]: ...
    def _report_ignoring_subs(self, name: str) -> None: ...
    def _extract_m3u8_formats(
        self,
        m3u8_url: str,
        video_id: str,
        ext: str | None = None,
        entry_protocol: str = "m3u8_native",
        preference: Any = None,
        quality: Any = None,
        m3u8_id: str | None = None,
        note: str | None = None,
        errnote: str | None = None,
        fatal: bool = True,
        live: bool = False,
        data: Any = None,
        headers: Mapping[str, Any] = ...,
        query: Mapping[str, Any] = ...,
    ) -> list[dict[str, Any]]: ...
    def _extract_m3u8_formats_and_subtitles(
        self,
        m3u8_url: str,
        video_id: str,
        ext: str | None = None,
        entry_protocol: str = "m3u8_native",
        preference: Any = None,
        quality: Any = None,
        m3u8_id: str | None = None,
        note: str | None = None,
        errnote: str | None = None,
        fatal: bool = True,
        live: bool = False,
        data: Any = None,
        headers: Mapping[str, Any] = {},
        query: Mapping[str, Any] = {},
    ) -> tuple[list[dict[str, Any]], list[dict[str, Any]]]: ...
    def _parse_m3u8_formats_and_subtitles(
        self,
        m3u8_doc: str,
        m3u8_url: str | None = None,
        ext: str | None = None,
        entry_protocol: str = "m3u8_native",
        preference: Any = None,
        quality: Any = None,
        m3u8_id: str | None = None,
        live: bool = False,
        note: str | None = None,
        errnote: str | None = None,
        fatal: bool = True,
        data: Any = None,
        headers: Mapping[str, Any] = {},
        query: Mapping[str, Any] = {},
        video_id: str | None = None,
    ) -> tuple[list[dict[str, Any]], list[dict[str, Any]]]: ...
    def _extract_m3u8_vod_duration(
        self,
        m3u8_vod_url: str,
        video_id: str,
        note: str | None = None,
        errnote: str | None = None,
        data: Any = None,
        headers: Mapping[str, Any] = {},
        query: Mapping[str, Any] = {},
    ) -> int | None: ...
    def _parse_m3u8_vod_duration(self, m3u8_vod: str, video_id: str) -> int: ...
    def _extract_mpd_vod_duration(
        self,
        mpd_url: str,
        video_id: str,
        note: str | None = None,
        errnote: str | None = None,
        data: Any = None,
        headers: Mapping[str, Any] = {},
        query: Mapping[str, Any] = {},
    ) -> int | None: ...
    @staticmethod
    def _xpath_ns(path: str, namespace: str | None = None) -> str: ...
    def _extract_smil_formats_and_subtitles(
        self,
        smil_url: str,
        video_id: str,
        fatal: bool = True,
        f4m_params: Mapping[str, Any] | None = None,
        transform_source: Callable[..., str] | None = None,
    ) -> tuple[list[dict[str, Any]], list[dict[str, Any]]]: ...
    def _extract_smil_formats(
        self,
        smil: ET.Element,
        smil_url: str,
        video_id: str,
        namespace: str | None = None,
        f4m_params: Mapping[str, Any] | None = None,
        transform_rtmp_url: Callable[[str, str], tuple[str, str]] | None = None,
    ) -> list[dict[str, Any]]: ...
    def _extract_smil_info(
        self, smil_url: str, video_id: str, fatal: bool = True, f4m_params: Mapping[str, Any] | None = None
    ) -> dict[str, Any]: ...
    def _download_smil(
        self, smil_url: str, video_id: str, fatal: bool = True, transform_source: Callable[..., str] | None = None
    ) -> ET.Element: ...
    def _parse_smil(
        self, smil: ET.Element, smil_url: str, video_id: str, f4m_params: Mapping[str, Any] | None = None
    ) -> dict[str, Any]: ...
    def _parse_smil_namespace(self, smil: str) -> str | None: ...
    def _parse_smil_formats(
        self,
        smil: ET.Element,
        smil_url: str,
        video_id: str,
        namespace: str | None = None,
        f4m_params: Mapping[str, Any] | None = None,
        transform_rtmp_url: Callable[[str, str], tuple[str, str]] | None = None,
    ) -> list[dict[str, Any]]: ...
    def _parse_smil_formats_and_subtitles(
        self,
        smil: ET.Element,
        smil_url: str,
        video_id: str,
        namespace: str | None = None,
        f4m_params: Mapping[str, Any] | None = None,
        transform_rtmp_url: Callable[[str, str], tuple[str, str]] | None = None,
    ) -> tuple[list[dict[str, Any]], list[dict[str, Any]]]: ...
    def _parse_smil_subtitles(
        self, smil: ET.Element, namespace: str | None = None, subtitles_lang: str = "en"
    ) -> list[dict[str, Any]]: ...
    def _extract_xspf_playlist(self, xspf_url: str, playlist_id: str, fatal: bool = True) -> list[dict[str, Any]]: ...
    def _parse_xspf(
        self, xspf_doc: ET.Element, playlist_id: str, xspf_url: str | None = None, xspf_base_url: str | None = None
    ) -> list[dict[str, Any]]: ...
    def _extract_mpd_formats(
        self,
        mpd_url: str,
        video_id: str,
        mpd_id: str | None = None,
        note: str | None = None,
        errnote: str | None = None,
        fatal: bool = True,
        data: Any = None,
        headers: Mapping[str, Any] = ...,
        query: Mapping[str, Any] = ...,
    ) -> list[dict[str, Any]]: ...
    def _extract_mpd_formats_and_subtitles(
        self,
        mpd_url: str,
        video_id: str,
        mpd_id: str | None = None,
        note: str | None = None,
        errnote: str | None = None,
        fatal: bool = True,
        data: Any = None,
        headers: Mapping[str, Any] = ...,
        query: Mapping[str, Any] = ...,
    ) -> tuple[list[dict[str, Any]], list[dict[str, Any]]]: ...
    def _extract_mpd_periods(
        self,
        mpd_url: str,
        video_id: str,
        mpd_id: str | None = None,
        note: str | None = None,
        errnote: str | None = None,
        fatal: bool = True,
        data: Any = None,
        headers: Mapping[str, Any] = {},
        query: Mapping[str, Any] = {},
    ) -> tuple[list[Any], dict[str, Any]]: ...
    def _parse_mpd_formats_and_subtitles(
        self,
        mpd_url: str,
        video_id: str,
        mpd_id: str | None = None,
        note: str | None = None,
        errnote: str | None = None,
        fatal: bool = True,
        data: Any = None,
        headers: Mapping[str, Any] = ...,
        query: Mapping[str, Any] = ...,
    ) -> tuple[list[dict[str, Any]], list[dict[str, Any]]]: ...
    def _merge_mpd_periods(self, periods: Iterable[Mapping[str, Any]]) -> tuple[list[Any], dict[str, Any]]: ...
    def _parse_mpd_periods(
        self, mpd_doc: ET.Element, mpd_id: str | None = None, mpd_base_url: str = "", mpd_url: str | None = None
    ) -> tuple[list[Any], dict[str, Any]]: ...
    def _extract_ism_formats(
        self,
        ism_url: str,
        video_id: str,
        ism_id: str | None = None,
        note: str | None = None,
        errnote: str | None = None,
        fatal: bool = True,
        data: Any = None,
        headers: Mapping[str, Any] = ...,
        query: Mapping[str, Any] = ...,
    ) -> list[dict[str, Any]]: ...
    def _extract_ism_formats_and_subtitles(
        self,
        ism_url: str,
        video_id: str,
        ism_id: str | None = None,
        note: str | None = None,
        errnote: str | None = None,
        fatal: bool = True,
        data: Any = None,
        headers: Mapping[str, Any] = {},
        query: Mapping[str, Any] = {},
    ) -> tuple[list[dict[str, Any]], list[dict[str, Any]]]: ...
    def _parse_ism_formats_and_subtitles(
        self, ism_doc: str, ism_url: str, ism_id: str | None = None
    ) -> tuple[list[dict[str, Any]], list[dict[str, Any]]]: ...
    def _parse_html5_media_entries(
        self,
        base_url: str,
        webpage: str,
        video_id: str,
        m3u8_id: str | None = None,
        m3u8_entry_protocol: str = "m3u8_native",
        mpd_id: str | None = None,
        preference: Any = None,
        quality: Any = None,
        _headers: Mapping[str, Any] | None = None,
    ) -> list[dict[str, Any]]: ...
    def _extract_akamai_formats(
        self, manifest_url: str, video_id: str, hosts: Mapping[str, Any] = ...
    ) -> list[dict[str, Any]]: ...
    def _extract_akamai_formats_and_subtitles(
        self, manifest_url: str, video_id: str, hosts: Mapping[str, Any] = {}
    ) -> tuple[list[dict[str, Any]], list[dict[str, Any]]]: ...
    def _extract_wowza_formats(
        self, url: str, video_id: str, m3u8_entry_protocol: str = "m3u8_native", skip_protocols: Collection[str] = []
    ) -> list[dict[str, Any]]: ...
    def _find_jwplayer_data(
        self, webpage: str, video_id: str | None = None, transform_source: Callable[..., str] = ...
    ) -> Any: ...
    def _extract_jwplayer_data(
        self,
        webpage: str,
        video_id: str,
        *args: Any,
        transform_source: Callable[..., str] = ...,
        require_title: bool = True,
        m3u8_id: str | None = None,
        mpd_id: str | None = None,
        rtmp_params: Mapping[str, Any] | None = None,
        base_url: str | None = None,
    ) -> list[dict[str, Any]]: ...
    def _parse_jwplayer_data(
        self,
        jwplayer_data: Mapping[str, Any],
        video_id: str | None = None,
        require_title: bool = True,
        m3u8_id: str | None = None,
        mpd_id: str | None = None,
        rtmp_params: Mapping[str, Any] | None = None,
        base_url: str | None = None,
    ) -> list[dict[str, Any]]: ...
    def _parse_jwplayer_formats(
        self,
        jwplayer_sources_data: Iterable[Mapping[str, Any]],
        video_id: str | None = None,
        m3u8_id: str | None = None,
        mpd_id: str | None = None,
        rtmp_params: Mapping[str, Any] | None = None,
        base_url: str | None = None,
    ) -> list[dict[str, Any]]: ...
    def _int(
        self,
        v: Any,
        name: str,
        fatal: bool = False,
        *,
        scale: int = 1,
        default: int | None = None,
        get_attr: str | None = None,
        invscale: int = 1,
        base: int | None = None,
    ) -> int | None: ...
    def _float(
        self, v: Any, name: str, fatal: bool = False, *, scale: int = 1, invscale: int = 1, default: float | None = None
    ) -> float | None: ...
    def _set_cookie(
        self,
        domain: str,
        name: str,
        value: str,
        expire_time: int | None = None,
        port: int | None = None,
        path: str = "/",
        secure: bool = False,
        discard: bool = False,
        rest: dict[str, Any] = {},
        **kwargs: Unused,
    ) -> None: ...
    def _live_title(self, name: _T) -> _T: ...
    def _get_cookies(self, url: str) -> LenientSimpleCookie: ...
    def _apply_first_set_cookie_header(self, url_handle: Response, cookie: str) -> None: ...
    @classproperty
    def _RETURN_TYPE(cls) -> str: ...
    def _get_subtitles(self, *args: Any, **kwargs: Any) -> list[dict[str, Any]]: ...  # Not implemented here.
    # Passes *args and **kwargs to _get_comments.
    def extract_comments(self, *args: Any, **kwargs: Any) -> list[dict[str, Any]]: ...
    def _get_comments(self, *args: Any, **kwargs: Any) -> list[dict[str, Any]]: ...  # Not implemented here.
    @staticmethod
    def _merge_subtitle_items(
        subtitle_list1: Iterable[Mapping[str, Any]], subtitle_list2: Iterable[Mapping[str, Any]]
    ) -> list[dict[str, Any]]: ...
    @classmethod
    def _merge_subtitles(cls, *dicts: dict[str, Any], target: Any = None) -> dict[str, Any]: ...
    # Calls _get_automatic_captions which only raises NotImplementedError here.
    def extract_automatic_captions(self, *args: Any, **kwargs: Any) -> dict[str, Any]: ...
    @cached_property
    def _cookies_passed(self) -> bool: ...
    def _mark_watched(self, *args: Any, **kwargs: Any) -> Any: ...  # Not implemented here.
    @staticmethod
    def _generic_id(url: str) -> str: ...
    def _generic_title(self, url: str = "", webpage: str = "", *, default: str | None = None) -> str | None: ...
    def _extract_chapters_helper(
        self,
        chapter_list: Iterable[Mapping[str, Any]],
        start_function: Callable[..., Any],
        title_function: Callable[..., Any],
        duration: float,
        strict: bool = True,
    ) -> list[dict[str, int]] | None: ...
    def _extract_chapters_from_description(
        self, description: str | None, duration: str | None
    ) -> list[dict[str, int]] | None: ...
    # Passes *args and **kwargs to _mark_watched which only raises NotImplementedError here.
    def mark_watched(self, *args: Any, **kwargs: Any) -> None: ...
    def geo_verification_headers(self) -> dict[str, str]: ...
    # kwargs passed to _error_callback.
    def RetryManager(self, *, _retries: int | None, _error_callback: Callable[..., Any], **kwargs: Any) -> _RetryManager: ...
    @classmethod
    def extract_from_webpage(cls, ydl: YoutubeDL, url: str, webpage: str) -> Iterator[_InfoDict]: ...
    def _yes_playlist(
        self,
        playlist_id: str,
        video_id: str,
        smuggled_data: Any = None,
        *,
        playlist_label: str = "playlist",
        video_label: str = "video",
    ) -> bool: ...
    def _error_or_warning(self, err: str, _count: int | None = None, _retries: int = 0, *, fatal: bool = True) -> None: ...
    def _extract_generic_embeds(
        self,
        url: str,
        *args: Unused,
        info_dict: _InfoDict = {},  # type: ignore[typeddict-item]  # pyright: ignore[reportArgumentType]
        note: str = "Extracting generic embeds",
        **kwargs: Unused,
    ) -> list[dict[str, Any]]: ...
    @classmethod
    def _extract_from_webpage(cls, url: str, webpage: str) -> Iterator[_InfoDict]: ...
    @classmethod
    def _extract_embed_urls(cls, url: str, webpage: str) -> Iterator[str]: ...
    @classmethod
    def _extract_url(cls, webpage: str) -> str | None: ...
    @classmethod
    def __init_subclass__(cls, *, plugin_name: str | None = None, **kwargs: Any) -> None: ...

    class StopExtraction(Exception): ...
    class CommentsDisabled(Exception): ...

class SearchInfoExtractor(InfoExtractor):
    def _real_extract(self, query: str) -> _InfoDict: ...
    def _get_n_results(self, query: str, n: int) -> list[_InfoDict]: ...
    def _search_results(self, query: str) -> list[_InfoDict]: ...
    @classproperty
    def SEARCH_KEY(self) -> str | None: ...

class UnsupportedURLIE(InfoExtractor):
    IE_DESC: ClassVar[bool]



from abc import abstractmethod
from collections.abc import Callable, Sequence
from typing import Any
from typing_extensions import Self

from wtforms.fields import HiddenField
from wtforms.fields.core import UnboundField, _Filter, _FormT, _Validator, _Widget
from wtforms.form import BaseForm
from wtforms.meta import DefaultMeta, _SupportsGettextAndNgettext

__all__ = ("CSRFTokenField", "CSRF")

class CSRFTokenField(HiddenField):
    current_token: str | None
    csrf_impl: CSRF
    def __init__(
        self,
        label: str | None = None,
        validators: tuple[_Validator[_FormT, Self], ...] | list[Any] | None = None,
        filters: Sequence[_Filter] = (),
        description: str = "",
        id: str | None = None,
        default: str | Callable[[], str] | None = None,
        widget: _Widget[Self] | None = None,
        render_kw: dict[str, Any] | None = None,
        name: str | None = None,
        _form: BaseForm | None = None,
        _prefix: str = "",
        _translations: _SupportsGettextAndNgettext | None = None,
        _meta: DefaultMeta | None = None,
        *,
        csrf_impl: CSRF,
    ) -> None: ...

class CSRF:
    field_class: type[CSRFTokenField]
    def setup_form(self, form: BaseForm) -> list[tuple[str, UnboundField[Any]]]: ...
    @abstractmethod
    def generate_csrf_token(self, csrf_token_field: CSRFTokenField) -> str: ...
    @abstractmethod
    def validate_csrf_token(self, form: BaseForm, field: CSRFTokenField) -> None: ...



from typing import Final

URL: Final = "url"
METHOD: Final = "method"
USER_AGENT: Final = "user_agent"
CLIENT_IP: Final = "client_ip"
X_FORWARDED_FOR: Final = "x_forwarded_for"
STATUS: Final = "status"
CONTENT_LENGTH: Final = "content_length"
XRAY_HEADER: Final = "X-Amzn-Trace-Id"
ALT_XRAY_HEADER: Final = "HTTP_X_AMZN_TRACE_ID"
request_keys: tuple[str, ...]
response_keys: tuple[str, ...]



"""Type-annotated versions of the recipes from the itertools docs.

These are all meant to be examples of idiomatic itertools usage,
so they should all type-check without error.
"""

from __future__ import annotations

import collections
import math
import operator
import sys
from itertools import chain, combinations, count, cycle, filterfalse, groupby, islice, product, repeat, starmap, tee, zip_longest
from typing import (
    Any,
    Callable,
    Collection,
    Hashable,
    Iterable,
    Iterator,
    Literal,
    Sequence,
    Tuple,
    Type,
    TypeVar,
    Union,
    overload,
)
from typing_extensions import TypeAlias, TypeVarTuple, Unpack

_T = TypeVar("_T")
_T1 = TypeVar("_T1")
_T2 = TypeVar("_T2")
_HashableT = TypeVar("_HashableT", bound=Hashable)
_Ts = TypeVarTuple("_Ts")


def take(n: int, iterable: Iterable[_T]) -> list[_T]:
    "Return first n items of the iterable as a list"
    return list(islice(iterable, n))


# Note: the itertools docs uses the parameter name "iterator",
# but the function actually accepts any iterable
# as its second argument
def prepend(value: _T1, iterator: Iterable[_T2]) -> Iterator[_T1 | _T2]:
    "Prepend a single value in front of an iterator"
    # prepend(1, [2, 3, 4]) --> 1 2 3 4
    return chain([value], iterator)


def tabulate(function: Callable[[int], _T], start: int = 0) -> Iterator[_T]:
    "Return function(0), function(1), ..."
    return map(function, count(start))


def repeatfunc(func: Callable[[Unpack[_Ts]], _T], times: int | None = None, *args: Unpack[_Ts]) -> Iterator[_T]:
    """Repeat calls to func with specified arguments.

    Example:  repeatfunc(random.random)
    """
    if times is None:
        return starmap(func, repeat(args))
    return starmap(func, repeat(args, times))


def flatten(list_of_lists: Iterable[Iterable[_T]]) -> Iterator[_T]:
    "Flatten one level of nesting"
    return chain.from_iterable(list_of_lists)


def ncycles(iterable: Iterable[_T], n: int) -> Iterator[_T]:
    "Returns the sequence elements n times"
    return chain.from_iterable(repeat(tuple(iterable), n))


def tail(n: int, iterable: Iterable[_T]) -> Iterator[_T]:
    "Return an iterator over the last n items"
    # tail(3, 'ABCDEFG') --> E F G
    return iter(collections.deque(iterable, maxlen=n))


# This function *accepts* any iterable,
# but it only *makes sense* to use it with an iterator
def consume(iterator: Iterator[object], n: int | None = None) -> None:
    "Advance the iterator n-steps ahead. If n is None, consume entirely."
    # Use functions that consume iterators at C speed.
    if n is None:
        # feed the entire iterator into a zero-length deque
        collections.deque(iterator, maxlen=0)
    else:
        # advance to the empty slice starting at position n
        next(islice(iterator, n, n), None)


@overload
def nth(iterable: Iterable[_T], n: int, default: None = None) -> _T | None: ...


@overload
def nth(iterable: Iterable[_T], n: int, default: _T1) -> _T | _T1: ...


def nth(iterable: Iterable[object], n: int, default: object = None) -> object:
    "Returns the nth item or a default value"
    return next(islice(iterable, n, None), default)


@overload
def quantify(iterable: Iterable[object]) -> int: ...


@overload
def quantify(iterable: Iterable[_T], pred: Callable[[_T], bool]) -> int: ...


def quantify(iterable: Iterable[object], pred: Callable[[Any], bool] = bool) -> int:
    "Given a predicate that returns True or False, count the True results."
    return sum(map(pred, iterable))


@overload
def first_true(
    iterable: Iterable[_T], default: Literal[False] = False, pred: Callable[[_T], bool] | None = None
) -> _T | Literal[False]: ...


@overload
def first_true(iterable: Iterable[_T], default: _T1, pred: Callable[[_T], bool] | None = None) -> _T | _T1: ...


def first_true(iterable: Iterable[object], default: object = False, pred: Callable[[Any], bool] | None = None) -> object:
    """Returns the first true value in the iterable.
    If no true value is found, returns *default*
    If *pred* is not None, returns the first item
    for which pred(item) is true.
    """
    # first_true([a,b,c], x) --> a or b or c or x
    # first_true([a,b], x, f) --> a if f(a) else b if f(b) else x
    return next(filter(pred, iterable), default)


_ExceptionOrExceptionTuple: TypeAlias = Union[Type[BaseException], Tuple[Type[BaseException], ...]]


@overload
def iter_except(func: Callable[[], _T], exception: _ExceptionOrExceptionTuple, first: None = None) -> Iterator[_T]: ...


@overload
def iter_except(
    func: Callable[[], _T], exception: _ExceptionOrExceptionTuple, first: Callable[[], _T1]
) -> Iterator[_T | _T1]: ...


def iter_except(
    func: Callable[[], object], exception: _ExceptionOrExceptionTuple, first: Callable[[], object] | None = None
) -> Iterator[object]:
    """Call a function repeatedly until an exception is raised.
    Converts a call-until-exception interface to an iterator interface.
    Like builtins.iter(func, sentinel) but uses an exception instead
    of a sentinel to end the loop.
    Examples:
        iter_except(functools.partial(heappop, h), IndexError)   # priority queue iterator
        iter_except(d.popitem, KeyError)                         # non-blocking dict iterator
        iter_except(d.popleft, IndexError)                       # non-blocking deque iterator
        iter_except(q.get_nowait, Queue.Empty)                   # loop over a producer Queue
        iter_except(s.pop, KeyError)                             # non-blocking set iterator
    """
    try:
        if first is not None:
            yield first()  # For database APIs needing an initial cast to db.first()
        while True:
            yield func()
    except exception:
        pass


def sliding_window(iterable: Iterable[_T], n: int) -> Iterator[tuple[_T, ...]]:
    # sliding_window('ABCDEFG', 4) --> ABCD BCDE CDEF DEFG
    it = iter(iterable)
    window = collections.deque(islice(it, n - 1), maxlen=n)
    for x in it:
        window.append(x)
        yield tuple(window)


def roundrobin(*iterables: Iterable[_T]) -> Iterator[_T]:
    "roundrobin('ABC', 'D', 'EF') --> A D E B F C"
    # Recipe credited to George Sakkis
    num_active = len(iterables)
    nexts: Iterator[Callable[[], _T]] = cycle(iter(it).__next__ for it in iterables)
    while num_active:
        try:
            for next in nexts:
                yield next()
        except StopIteration:
            # Remove the iterator we just exhausted from the cycle.
            num_active -= 1
            nexts = cycle(islice(nexts, num_active))


def partition(pred: Callable[[_T], bool], iterable: Iterable[_T]) -> tuple[Iterator[_T], Iterator[_T]]:
    """Partition entries into false entries and true entries.
    If *pred* is slow, consider wrapping it with functools.lru_cache().
    """
    # partition(is_odd, range(10)) --> 0 2 4 6 8   and  1 3 5 7 9
    t1, t2 = tee(iterable)
    return filterfalse(pred, t1), filter(pred, t2)


def subslices(seq: Sequence[_T]) -> Iterator[Sequence[_T]]:
    "Return all contiguous non-empty subslices of a sequence"
    # subslices('ABCD') --> A AB ABC ABCD B BC BCD C CD D
    slices = starmap(slice, combinations(range(len(seq) + 1), 2))
    return map(operator.getitem, repeat(seq), slices)


def before_and_after(predicate: Callable[[_T], bool], it: Iterable[_T]) -> tuple[Iterator[_T], Iterator[_T]]:
    """Variant of takewhile() that allows complete
    access to the remainder of the iterator.
    >>> it = iter('ABCdEfGhI')
    >>> all_upper, remainder = before_and_after(str.isupper, it)
    >>> ''.join(all_upper)
    'ABC'
    >>> ''.join(remainder)     # takewhile() would lose the 'd'
    'dEfGhI'
    Note that the first iterator must be fully
    consumed before the second iterator can
    generate valid results.
    """
    it = iter(it)
    transition: list[_T] = []

    def true_iterator() -> Iterator[_T]:
        for elem in it:
            if predicate(elem):
                yield elem
            else:
                transition.append(elem)
                return

    def remainder_iterator() -> Iterator[_T]:
        yield from transition
        yield from it

    return true_iterator(), remainder_iterator()


@overload
def unique_everseen(iterable: Iterable[_HashableT], key: None = None) -> Iterator[_HashableT]: ...


@overload
def unique_everseen(iterable: Iterable[_T], key: Callable[[_T], Hashable]) -> Iterator[_T]: ...


def unique_everseen(iterable: Iterable[_T], key: Callable[[_T], Hashable] | None = None) -> Iterator[_T]:
    "List unique elements, preserving order. Remember all elements ever seen."
    # unique_everseen('AAAABBBCCDAABBB') --> A B C D
    # unique_everseen('ABBcCAD', str.lower) --> A B c D
    seen: set[Hashable] = set()
    if key is None:
        for element in filterfalse(seen.__contains__, iterable):
            seen.add(element)
            yield element
        # For order preserving deduplication,
        # a faster but non-lazy solution is:
        #     yield from dict.fromkeys(iterable)
    else:
        for element in iterable:
            k = key(element)
            if k not in seen:
                seen.add(k)
                yield element
        # For use cases that allow the last matching element to be returned,
        # a faster but non-lazy solution is:
        #      t1, t2 = tee(iterable)
        #      yield from dict(zip(map(key, t1), t2)).values()


# Slightly adapted from the docs recipe; a one-liner was a bit much for pyright
def unique_justseen(iterable: Iterable[_T], key: Callable[[_T], bool] | None = None) -> Iterator[_T]:
    "List unique elements, preserving order. Remember only the element just seen."
    # unique_justseen('AAAABBBCCDAABBB') --> A B C D A B
    # unique_justseen('ABBcCAD', str.lower) --> A B c A D
    if key is None:
        return map(operator.itemgetter(0), groupby(iterable))
    g: groupby[_T | bool, _T] = groupby(iterable, key)
    return map(next, map(operator.itemgetter(1), g))


def powerset(iterable: Iterable[_T]) -> Iterator[tuple[_T, ...]]:
    "powerset([1,2,3]) --> () (1,) (2,) (3,) (1,2) (1,3) (2,3) (1,2,3)"
    s = list(iterable)
    return chain.from_iterable(combinations(s, r) for r in range(len(s) + 1))


def polynomial_derivative(coefficients: Sequence[float]) -> list[float]:
    """Compute the first derivative of a polynomial.
    f(x)  =  x³ -4x² -17x + 60
    f'(x) = 3x² -8x  -17
    """
    # polynomial_derivative([1, -4, -17, 60]) -> [3, -8, -17]
    n = len(coefficients)
    powers = reversed(range(1, n))
    return list(map(operator.mul, coefficients, powers))


def nth_combination(iterable: Iterable[_T], r: int, index: int) -> tuple[_T, ...]:
    "Equivalent to list(combinations(iterable, r))[index]"
    pool = tuple(iterable)
    n = len(pool)
    c = math.comb(n, r)
    if index < 0:
        index += c
    if index < 0 or index >= c:
        raise IndexError
    result: list[_T] = []
    while r:
        c, n, r = c * r // n, n - 1, r - 1
        while index >= c:
            index -= c
            c, n = c * (n - r) // n, n - 1
        result.append(pool[-1 - n])
    return tuple(result)


if sys.version_info >= (3, 10):

    @overload
    def grouper(
        iterable: Iterable[_T], n: int, *, incomplete: Literal["fill"] = "fill", fillvalue: None = None
    ) -> Iterator[tuple[_T | None, ...]]: ...

    @overload
    def grouper(
        iterable: Iterable[_T], n: int, *, incomplete: Literal["fill"] = "fill", fillvalue: _T1
    ) -> Iterator[tuple[_T | _T1, ...]]: ...

    @overload
    def grouper(
        iterable: Iterable[_T], n: int, *, incomplete: Literal["strict", "ignore"], fillvalue: None = None
    ) -> Iterator[tuple[_T, ...]]: ...

    def grouper(
        iterable: Iterable[object], n: int, *, incomplete: Literal["fill", "strict", "ignore"] = "fill", fillvalue: object = None
    ) -> Iterator[tuple[object, ...]]:
        "Collect data into non-overlapping fixed-length chunks or blocks"
        # grouper('ABCDEFG', 3, fillvalue='x') --> ABC DEF Gxx
        # grouper('ABCDEFG', 3, incomplete='strict') --> ABC DEF ValueError
        # grouper('ABCDEFG', 3, incomplete='ignore') --> ABC DEF
        args = [iter(iterable)] * n
        if incomplete == "fill":
            return zip_longest(*args, fillvalue=fillvalue)
        if incomplete == "strict":
            return zip(*args, strict=True)
        if incomplete == "ignore":
            return zip(*args)
        else:
            raise ValueError("Expected fill, strict, or ignore")

    def transpose(it: Iterable[Iterable[_T]]) -> Iterator[tuple[_T, ...]]:
        "Swap the rows and columns of the input."
        # transpose([(1, 2, 3), (11, 22, 33)]) --> (1, 11) (2, 22) (3, 33)
        return zip(*it, strict=True)


if sys.version_info >= (3, 12):
    from itertools import batched

    def sum_of_squares(it: Iterable[float]) -> float:
        "Add up the squares of the input values."
        # sum_of_squares([10, 20, 30]) -> 1400
        return math.sumprod(*tee(it))

    def convolve(signal: Iterable[float], kernel: Iterable[float]) -> Iterator[float]:
        """Discrete linear convolution of two iterables.
        The kernel is fully consumed before the calculations begin.
        The signal is consumed lazily and can be infinite.
        Convolutions are mathematically commutative.
        If the signal and kernel are swapped,
        the output will be the same.
        Article:  https://betterexplained.com/articles/intuitive-convolution/
        Video:    https://www.youtube.com/watch?v=KuXjwB4LzSA
        """
        # convolve(data, [0.25, 0.25, 0.25, 0.25]) --> Moving average (blur)
        # convolve(data, [1/2, 0, -1/2]) --> 1st derivative estimate
        # convolve(data, [1, -2, 1]) --> 2nd derivative estimate
        kernel = tuple(kernel)[::-1]
        n = len(kernel)
        padded_signal = chain(repeat(0, n - 1), signal, repeat(0, n - 1))
        windowed_signal = sliding_window(padded_signal, n)
        return map(math.sumprod, repeat(kernel), windowed_signal)

    def polynomial_eval(coefficients: Sequence[float], x: float) -> float:
        """Evaluate a polynomial at a specific value.
        Computes with better numeric stability than Horner's method.
        """
        # Evaluate x³ -4x² -17x + 60 at x = 2.5
        # polynomial_eval([1, -4, -17, 60], x=2.5) --> 8.125
        n = len(coefficients)
        if not n:
            return type(x)(0)
        powers = map(pow, repeat(x), reversed(range(n)))
        return math.sumprod(coefficients, powers)

    def matmul(m1: Sequence[Collection[float]], m2: Sequence[Collection[float]]) -> Iterator[tuple[float, ...]]:
        "Multiply two matrices."
        # matmul([(7, 5), (3, 5)], [(2, 5), (7, 9)]) --> (49, 80), (41, 60)
        n = len(m2[0])
        return batched(starmap(math.sumprod, product(m1, transpose(m2))), n)



from collections.abc import Iterator
from typing import overload
from typing_extensions import Self

__all__ = ["Range", "ContentRange"]

class Range:
    start: int | None
    end: int | None
    @overload
    def __init__(self, start: None, end: None) -> None: ...
    @overload
    def __init__(self, start: int, end: int | None) -> None: ...
    def range_for_length(self, length: int | None) -> tuple[int, int] | None: ...
    def content_range(self, length: int | None) -> ContentRange | None: ...
    def __iter__(self) -> Iterator[int | None]: ...
    @classmethod
    def parse(cls, header: str | None) -> Self | None: ...

class ContentRange:
    start: int | None
    stop: int | None
    length: int | None
    @overload
    def __init__(self, start: None, stop: None, length: int | None) -> None: ...
    @overload
    def __init__(self, start: int, stop: int, length: int | None) -> None: ...
    def __iter__(self) -> Iterator[int | None]: ...
    @classmethod
    def parse(cls, value: str | None) -> Self | None: ...



"""Abstract class for IR Passes for Jac."""

from __future__ import annotations

from threading import Event
from typing import TYPE_CHECKING, TypeVar

import jaclang.pycore.unitree as uni
from jaclang.pycore.passes.transform import Transform

if TYPE_CHECKING:
    from jaclang.pycore.program import JacProgram

T = TypeVar("T", bound=uni.UniNode)


class UniPass(Transform[uni.Module, uni.Module]):
    """Abstract class for IR passes."""

    def __init__(
        self,
        ir_in: uni.Module,
        prog: JacProgram,
        cancel_token: Event | None = None,
    ) -> None:
        """Initialize parser."""
        self.term_signal = False
        self.prune_signal = False
        Transform.__init__(self, ir_in, prog, cancel_token=cancel_token)

    def before_pass(self) -> None:
        """Run once before pass."""

    def after_pass(self) -> None:
        """Run once after pass."""

    def enter_node(self, node: uni.UniNode) -> None:
        """Run on entering node."""
        from jaclang.pycore.helpers import pascal_to_snake

        if hasattr(self, f"enter_{pascal_to_snake(type(node).__name__)}"):
            getattr(self, f"enter_{pascal_to_snake(type(node).__name__)}")(node)

    def exit_node(self, node: uni.UniNode) -> None:
        """Run on exiting node."""
        from jaclang.pycore.helpers import pascal_to_snake

        if hasattr(self, f"exit_{pascal_to_snake(type(node).__name__)}"):
            getattr(self, f"exit_{pascal_to_snake(type(node).__name__)}")(node)

    def prune(self) -> None:
        """Prune traversal."""
        self.prune_signal = True

    @staticmethod
    def get_all_sub_nodes(
        node: uni.UniNode, typ: type[T], brute_force: bool = False
    ) -> list[T]:
        """Get all sub nodes of type."""
        result: list[T] = []
        # Assumes pass built the sub node table
        if not node:
            return result
        elif len(node._sub_node_tab):
            if typ in node._sub_node_tab:
                for i in node._sub_node_tab[typ]:
                    if isinstance(i, typ):
                        result.append(i)
        elif len(node.kid):
            if not brute_force:
                raise ValueError(f"Node has no sub_node_tab. {node}")
            # Brute force search
            else:
                for i in node.kid:
                    if isinstance(i, typ):
                        result.append(i)
                    result.extend(UniPass.get_all_sub_nodes(i, typ, brute_force))
        return result

    @staticmethod
    def find_parent_of_type(node: uni.UniNode, typ: type[T]) -> T | None:
        """Check if node has parent of type."""
        while node.parent:
            if isinstance(node.parent, typ):
                return node.parent
            node = node.parent
        return None

    @staticmethod
    def has_parent_of_node(node: uni.UniNode, parent: uni.UniNode) -> bool:
        """Check if node has parent of type."""
        while node.parent:
            if node.parent == parent:
                return True
            node = node.parent
        return False

    def recalculate_parents(self, node: uni.UniNode) -> None:
        """Recalculate parents."""
        if not node:
            return
        for i in node.kid:
            if i:
                i.parent = node
                self.recalculate_parents(i)

    # Transform Implementations
    # -------------------------
    def transform(self, ir_in: uni.Module) -> uni.Module:
        """Run pass."""
        # Only performs passes on proper ASTs
        self.ir_out = ir_in  # TODO: this should go away and just be orig
        if not isinstance(ir_in, uni.UniNode):
            return ir_in
        self.before_pass()
        if not isinstance(ir_in, uni.UniNode):
            raise ValueError("Current node is not an UniNode.")
        self.traverse(ir_in)
        self.after_pass()
        return self.ir_in

    def traverse(self, node: uni.UniNode) -> uni.UniNode:
        """Traverse tree."""
        if self.is_canceled():
            return node
        self.cur_node = node
        self.enter_node(node)
        if not self.prune_signal:
            for i in node.kid:
                if i:
                    self.traverse(i)
        else:
            self.prune_signal = False
        self.cur_node = node
        if self.is_canceled():
            return node
        self.exit_node(node)
        return node



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import typing

import google.protobuf.descriptor
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class AllocationDescription(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REQUESTED_BYTES_FIELD_NUMBER: builtins.int
    ALLOCATED_BYTES_FIELD_NUMBER: builtins.int
    ALLOCATOR_NAME_FIELD_NUMBER: builtins.int
    ALLOCATION_ID_FIELD_NUMBER: builtins.int
    HAS_SINGLE_REFERENCE_FIELD_NUMBER: builtins.int
    PTR_FIELD_NUMBER: builtins.int
    requested_bytes: builtins.int
    """Total number of bytes requested"""
    allocated_bytes: builtins.int
    """Total number of bytes allocated if known"""
    allocator_name: builtins.str
    """Name of the allocator used"""
    allocation_id: builtins.int
    """Identifier of the allocated buffer if known"""
    has_single_reference: builtins.bool
    """Set if this tensor only has one remaining reference"""
    ptr: builtins.int
    """Address of the allocation."""
    def __init__(
        self,
        *,
        requested_bytes: builtins.int | None = ...,
        allocated_bytes: builtins.int | None = ...,
        allocator_name: builtins.str | None = ...,
        allocation_id: builtins.int | None = ...,
        has_single_reference: builtins.bool | None = ...,
        ptr: builtins.int | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "allocated_bytes",
            b"allocated_bytes",
            "allocation_id",
            b"allocation_id",
            "allocator_name",
            b"allocator_name",
            "has_single_reference",
            b"has_single_reference",
            "ptr",
            b"ptr",
            "requested_bytes",
            b"requested_bytes",
        ],
    ) -> None: ...

global___AllocationDescription = AllocationDescription



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from numpy.random import RandomState

__all__ = ["random_reference", "lattice_reference", "sigma", "omega"]

@_dispatchable
def random_reference(G: Graph[_Node], niter: int = 1, connectivity: bool = True, seed: int | RandomState | None = None): ...
@_dispatchable
def lattice_reference(
    G: Graph[_Node], niter: int = 5, D=None, connectivity: bool = True, seed: int | RandomState | None = None
): ...
@_dispatchable
def sigma(G: Graph[_Node], niter: int = 100, nrand: int = 10, seed: int | RandomState | None = None) -> float: ...
@_dispatchable
def omega(G: Graph[_Node], niter: int = 5, nrand: int = 10, seed: int | RandomState | None = None) -> float: ...



from typing import Final

XK_ISO_Lock: Final = 0xFE01
XK_ISO_Level2_Latch: Final = 0xFE02
XK_ISO_Level3_Shift: Final = 0xFE03
XK_ISO_Level3_Latch: Final = 0xFE04
XK_ISO_Level3_Lock: Final = 0xFE05
XK_ISO_Group_Shift: Final = 0xFF7E
XK_ISO_Group_Latch: Final = 0xFE06
XK_ISO_Group_Lock: Final = 0xFE07
XK_ISO_Next_Group: Final = 0xFE08
XK_ISO_Next_Group_Lock: Final = 0xFE09
XK_ISO_Prev_Group: Final = 0xFE0A
XK_ISO_Prev_Group_Lock: Final = 0xFE0B
XK_ISO_First_Group: Final = 0xFE0C
XK_ISO_First_Group_Lock: Final = 0xFE0D
XK_ISO_Last_Group: Final = 0xFE0E
XK_ISO_Last_Group_Lock: Final = 0xFE0F
XK_ISO_Left_Tab: Final = 0xFE20
XK_ISO_Move_Line_Up: Final = 0xFE21
XK_ISO_Move_Line_Down: Final = 0xFE22
XK_ISO_Partial_Line_Up: Final = 0xFE23
XK_ISO_Partial_Line_Down: Final = 0xFE24
XK_ISO_Partial_Space_Left: Final = 0xFE25
XK_ISO_Partial_Space_Right: Final = 0xFE26
XK_ISO_Set_Margin_Left: Final = 0xFE27
XK_ISO_Set_Margin_Right: Final = 0xFE28
XK_ISO_Release_Margin_Left: Final = 0xFE29
XK_ISO_Release_Margin_Right: Final = 0xFE2A
XK_ISO_Release_Both_Margins: Final = 0xFE2B
XK_ISO_Fast_Cursor_Left: Final = 0xFE2C
XK_ISO_Fast_Cursor_Right: Final = 0xFE2D
XK_ISO_Fast_Cursor_Up: Final = 0xFE2E
XK_ISO_Fast_Cursor_Down: Final = 0xFE2F
XK_ISO_Continuous_Underline: Final = 0xFE30
XK_ISO_Discontinuous_Underline: Final = 0xFE31
XK_ISO_Emphasize: Final = 0xFE32
XK_ISO_Center_Object: Final = 0xFE33
XK_ISO_Enter: Final = 0xFE34
XK_dead_grave: Final = 0xFE50
XK_dead_acute: Final = 0xFE51
XK_dead_circumflex: Final = 0xFE52
XK_dead_tilde: Final = 0xFE53
XK_dead_macron: Final = 0xFE54
XK_dead_breve: Final = 0xFE55
XK_dead_abovedot: Final = 0xFE56
XK_dead_diaeresis: Final = 0xFE57
XK_dead_abovering: Final = 0xFE58
XK_dead_doubleacute: Final = 0xFE59
XK_dead_caron: Final = 0xFE5A
XK_dead_cedilla: Final = 0xFE5B
XK_dead_ogonek: Final = 0xFE5C
XK_dead_iota: Final = 0xFE5D
XK_dead_voiced_sound: Final = 0xFE5E
XK_dead_semivoiced_sound: Final = 0xFE5F
XK_dead_belowdot: Final = 0xFE60
XK_First_Virtual_Screen: Final = 0xFED0
XK_Prev_Virtual_Screen: Final = 0xFED1
XK_Next_Virtual_Screen: Final = 0xFED2
XK_Last_Virtual_Screen: Final = 0xFED4
XK_Terminate_Server: Final = 0xFED5
XK_AccessX_Enable: Final = 0xFE70
XK_AccessX_Feedback_Enable: Final = 0xFE71
XK_RepeatKeys_Enable: Final = 0xFE72
XK_SlowKeys_Enable: Final = 0xFE73
XK_BounceKeys_Enable: Final = 0xFE74
XK_StickyKeys_Enable: Final = 0xFE75
XK_MouseKeys_Enable: Final = 0xFE76
XK_MouseKeys_Accel_Enable: Final = 0xFE77
XK_Overlay1_Enable: Final = 0xFE78
XK_Overlay2_Enable: Final = 0xFE79
XK_AudibleBell_Enable: Final = 0xFE7A
XK_Pointer_Left: Final = 0xFEE0
XK_Pointer_Right: Final = 0xFEE1
XK_Pointer_Up: Final = 0xFEE2
XK_Pointer_Down: Final = 0xFEE3
XK_Pointer_UpLeft: Final = 0xFEE4
XK_Pointer_UpRight: Final = 0xFEE5
XK_Pointer_DownLeft: Final = 0xFEE6
XK_Pointer_DownRight: Final = 0xFEE7
XK_Pointer_Button_Dflt: Final = 0xFEE8
XK_Pointer_Button1: Final = 0xFEE9
XK_Pointer_Button2: Final = 0xFEEA
XK_Pointer_Button3: Final = 0xFEEB
XK_Pointer_Button4: Final = 0xFEEC
XK_Pointer_Button5: Final = 0xFEED
XK_Pointer_DblClick_Dflt: Final = 0xFEEE
XK_Pointer_DblClick1: Final = 0xFEEF
XK_Pointer_DblClick2: Final = 0xFEF0
XK_Pointer_DblClick3: Final = 0xFEF1
XK_Pointer_DblClick4: Final = 0xFEF2
XK_Pointer_DblClick5: Final = 0xFEF3
XK_Pointer_Drag_Dflt: Final = 0xFEF4
XK_Pointer_Drag1: Final = 0xFEF5
XK_Pointer_Drag2: Final = 0xFEF6
XK_Pointer_Drag3: Final = 0xFEF7
XK_Pointer_Drag4: Final = 0xFEF8
XK_Pointer_Drag5: Final = 0xFEFD
XK_Pointer_EnableKeys: Final = 0xFEF9
XK_Pointer_Accelerate: Final = 0xFEFA
XK_Pointer_DfltBtnNext: Final = 0xFEFB
XK_Pointer_DfltBtnPrev: Final = 0xFEFC



from _typeshed import (
    Incomplete,
    OpenBinaryModeReading,
    OpenBinaryModeWriting,
    OpenTextModeReading,
    OpenTextModeWriting,
    SupportsWrite,
    Unused,
)
from re import Pattern
from typing import IO, Any, ClassVar, Final, Generic, Literal, TextIO, TypeVar
from typing_extensions import deprecated

from docutils import TransformSpec, nodes

__docformat__: Final = "reStructuredText"

class InputError(OSError): ...
class OutputError(OSError): ...

def check_encoding(stream: TextIO, encoding: str) -> bool | None: ...
def error_string(err: BaseException) -> str: ...

_S = TypeVar("_S")

class Input(TransformSpec, Generic[_S]):
    component_type: ClassVar[str]
    default_source_path: ClassVar[str | None]
    encoding: str | None
    error_handler: str
    source: _S | None
    source_path: str | None
    successful_encoding: str | None = None
    def __init__(
        self,
        source: _S | None = None,
        source_path: str | None = None,
        encoding: str | None = "utf-8",
        error_handler: str = "strict",
    ) -> None: ...
    def read(self) -> str: ...
    def decode(self, data: str | bytes | bytearray) -> str: ...
    coding_slug: ClassVar[Pattern[bytes]]
    byte_order_marks: ClassVar[tuple[tuple[bytes, str], ...]]
    @deprecated("Deprecated and will be removed in Docutils 1.0.")
    def determine_encoding_from_data(self, data: str | bytes | bytearray) -> str | None: ...
    def isatty(self) -> bool: ...

class Output(TransformSpec):
    component_type: ClassVar[str]
    default_destination_path: ClassVar[str | None]
    encoding: Incomplete
    error_handler: Incomplete
    destination: Incomplete
    destination_path: Incomplete
    def __init__(
        self, destination=None, destination_path=None, encoding: str | None = None, error_handler: str = "strict"
    ) -> None: ...
    def write(self, data: str) -> Any: ...  # returns bytes or str
    def encode(self, data: str) -> Any: ...  # returns bytes or str

class ErrorOutput:
    destination: Incomplete
    encoding: Incomplete
    encoding_errors: Incomplete
    decoding_errors: Incomplete
    def __init__(
        self,
        destination: str | SupportsWrite[str] | SupportsWrite[bytes] | Literal[False] | None = None,
        encoding: str | None = None,
        encoding_errors: str = "backslashreplace",
        decoding_errors: str = "replace",
    ) -> None: ...
    def write(self, data: str | bytes | Exception) -> None: ...
    def close(self) -> None: ...
    def isatty(self) -> bool: ...

class FileInput(Input[IO[str]]):
    autoclose: bool
    def __init__(
        self,
        source=None,
        source_path=None,
        encoding: str | None = "utf-8",
        error_handler: str = "strict",
        autoclose: bool = True,
        mode: OpenTextModeReading | OpenBinaryModeReading = "r",
    ) -> None: ...
    def read(self) -> str: ...
    def readlines(self) -> list[str]: ...
    def close(self) -> None: ...

class FileOutput(Output):
    default_destination_path: ClassVar[str]
    mode: ClassVar[OpenTextModeWriting | OpenBinaryModeWriting]
    opened: bool
    autoclose: Incomplete
    destination: Incomplete
    destination_path: Incomplete
    def __init__(
        self,
        destination=None,
        destination_path=None,
        encoding=None,
        error_handler: str = "strict",
        autoclose: bool = True,
        handle_io_errors=None,
        mode=None,
    ) -> None: ...
    def open(self) -> None: ...
    def write(self, data): ...
    def close(self) -> None: ...

@deprecated("The `BinaryFileOutput` is deprecated by `FileOutput` and will be removed in Docutils 0.24.")
class BinaryFileOutput(FileOutput): ...

class StringInput(Input[str]):
    default_source_path: ClassVar[str]
    def read(self): ...

class StringOutput(Output):
    default_destination_path: ClassVar[str]
    destination: str | bytes  # only defined after call to write()
    def write(self, data): ...

class NullInput(Input[Any]):
    default_source_path: ClassVar[str]
    def read(self) -> str: ...

class NullOutput(Output):
    default_destination_path: ClassVar[str]
    def write(self, data: Unused) -> None: ...

class DocTreeInput(Input[nodes.document]):
    default_source_path: ClassVar[str]
    def read(self): ...



from _typeshed import StrOrBytesPath
from collections.abc import Iterable

def rmtree(path: StrOrBytesPath) -> None: ...
def isdir(path: StrOrBytesPath) -> bool: ...
def listdir(path: StrOrBytesPath) -> list[str]: ...
def exists(path: StrOrBytesPath) -> bool: ...
def copy(src: StrOrBytesPath, dst: StrOrBytesPath, overwrite: bool = False) -> None: ...
def makedirs(path: StrOrBytesPath) -> None: ...
def glob(pattern: str | bytes | Iterable[str | bytes]) -> list[str]: ...
def __getattr__(name: str): ...  # incomplete module



from _typeshed import Incomplete
from collections.abc import Callable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

from .edmondskarp import edmonds_karp

__all__ = ["gomory_hu_tree"]
default_flow_func = edmonds_karp

@_dispatchable
def gomory_hu_tree(G: Graph[_Node], capacity: str = "capacity", flow_func: Callable[..., Incomplete] | None = None): ...



from typing import Final
from xml.dom.pulldom import DOMEventStream
from xml.sax import _SupportsReadClose
from xml.sax.xmlreader import XMLReader

__origin__: Final = "xml.dom.pulldom"

def parse(
    stream_or_string: str | _SupportsReadClose[str] | _SupportsReadClose[bytes],
    parser: XMLReader | None = None,
    bufsize: int | None = None,
    forbid_dtd: bool = False,
    forbid_entities: bool = True,
    forbid_external: bool = True,
) -> DOMEventStream: ...
def parseString(
    string: str,
    parser: XMLReader | None = None,
    forbid_dtd: bool = False,
    forbid_entities: bool = True,
    forbid_external: bool = True,
) -> DOMEventStream: ...



from grpc_channelz.v1 import channelz_pb2, channelz_pb2_grpc

class ChannelzServicer(channelz_pb2_grpc.ChannelzServicer):
    @staticmethod
    async def GetTopChannels(request: channelz_pb2.GetTopChannelsRequest, context) -> channelz_pb2.GetTopChannelsResponse: ...
    @staticmethod
    async def GetServers(request: channelz_pb2.GetServersRequest, context) -> channelz_pb2.GetServersResponse: ...
    @staticmethod
    async def GetServer(request: channelz_pb2.GetServerRequest, context) -> channelz_pb2.GetServerResponse: ...
    @staticmethod
    async def GetServerSockets(
        request: channelz_pb2.GetServerSocketsRequest, context
    ) -> channelz_pb2.GetServerSocketsResponse: ...
    @staticmethod
    async def GetChannel(request: channelz_pb2.GetChannelRequest, context) -> channelz_pb2.GetChannelResponse: ...
    @staticmethod
    async def GetSubchannel(request: channelz_pb2.GetSubchannelRequest, context) -> channelz_pb2.GetSubchannelResponse: ...
    @staticmethod
    async def GetSocket(request: channelz_pb2.GetSocketRequest, context) -> channelz_pb2.GetSocketResponse: ...



from typing import ClassVar

from . import base

class Compiler(base.Compiler):
    src_extensions: ClassVar[list[str]]
    obj_extension: ClassVar[str]
    static_lib_extension: ClassVar[str]
    shared_lib_extension: ClassVar[str]
    dylib_lib_extension: ClassVar[str]
    xcode_stub_lib_extension: ClassVar[str]
    static_lib_format: ClassVar[str]
    shared_lib_format: ClassVar[str]
    dylib_lib_format: ClassVar[str]
    xcode_stub_lib_format: ClassVar[str]
    def runtime_library_dir_option(self, dir: str) -> str | list[str]: ...  # type: ignore[override]



import logging
from collections.abc import Callable, Generator
from typing import Any
from typing_extensions import Self

from .base import BaseMixin
from .collection import CollectionMixin
from .contains import ContainsMixin
from .date import DateMixin
from .dict import DictMixin
from .dynamic import DynamicMixin
from .exception import ExceptionMixin
from .extracting import ExtractingMixin
from .file import FileMixin
from .helpers import HelpersMixin
from .numeric import NumericMixin
from .snapshot import SnapshotMixin
from .string import StringMixin

__version__: str
__tracebackhide__: bool

class WarningLoggingAdapter(logging.LoggerAdapter[logging.Logger]):
    def process(self, msg: str, kwargs: Any) -> tuple[str, Any]: ...

class AssertionBuilder(
    StringMixin,
    SnapshotMixin,
    NumericMixin,
    HelpersMixin,
    FileMixin,
    ExtractingMixin,
    ExceptionMixin,
    DynamicMixin,
    DictMixin,
    DateMixin,
    ContainsMixin,
    CollectionMixin,
    BaseMixin,
):
    val: Any
    description: str
    kind: str | None
    expected: BaseException | None
    logger: logging.Logger
    def __init__(
        self,
        val: Any,
        description: str = "",
        kind: str | None = None,
        expected: BaseException | None = None,
        logger: logging.Logger | None = None,
    ) -> None: ...
    def builder(
        self,
        val: Any,
        description: str = "",
        kind: str | None = None,
        expected: BaseException | None = None,
        logger: logging.Logger | None = None,
    ) -> Self: ...
    def error(self, msg: str) -> Self: ...

def soft_assertions() -> Generator[None, None, None]: ...
def assert_that(val: Any, description: str = "") -> AssertionBuilder: ...
def assert_warn(val: Any, description: str = "", logger: logging.Logger | None = None) -> AssertionBuilder: ...
def fail(msg: str = "") -> None: ...
def soft_fail(msg: str = "") -> None: ...
def add_extension(func: Callable[[AssertionBuilder], AssertionBuilder]) -> None: ...
def remove_extension(func: Callable[[AssertionBuilder], AssertionBuilder]) -> None: ...



from _typeshed import Incomplete
from collections.abc import Iterable

from . import base

spaceCharacters: str

class Filter(base.Filter[dict[str, Incomplete]]):
    require_matching_tags: bool
    def __init__(self, source: Iterable[dict[str, Incomplete]], require_matching_tags: bool = True) -> None: ...



import datetime

from ..core import WesternCalendar

class Ireland(WesternCalendar):
    def get_june_holiday(self, year: int) -> tuple[datetime.date, str]: ...
    def get_august_holiday(self, year: int) -> tuple[datetime.date, str]: ...



from .oauth1_auth import OAuth1 as OAuth1
from .oauth1_session import OAuth1Session as OAuth1Session
from .oauth2_auth import OAuth2 as OAuth2
from .oauth2_session import OAuth2Session as OAuth2Session, TokenUpdated as TokenUpdated

__version__: str



from contextlib import AbstractContextManager
from typing import Any, TextIO, TypeVar

from .ansitowin32 import StreamWrapper

_TextIOT = TypeVar("_TextIOT", bound=TextIO)

orig_stdout: TextIO | None
orig_stderr: TextIO | None
wrapped_stdout: TextIO | StreamWrapper
wrapped_stderr: TextIO | StreamWrapper
atexit_done: bool
fixed_windows_console: bool

def reset_all() -> None: ...
def init(autoreset: bool = False, convert: bool | None = None, strip: bool | None = None, wrap: bool = True) -> None: ...
def deinit() -> None: ...
def colorama_text(*args: Any, **kwargs: Any) -> AbstractContextManager[None]: ...
def reinit() -> None: ...
def wrap_stream(
    stream: _TextIOT, convert: bool | None, strip: bool | None, autoreset: bool, wrap: bool
) -> _TextIOT | StreamWrapper: ...
def just_fix_windows_console() -> None: ...



from .assertpy import (
    WarningLoggingAdapter as WarningLoggingAdapter,
    __version__ as __version__,
    add_extension as add_extension,
    assert_that as assert_that,
    assert_warn as assert_warn,
    fail as fail,
    remove_extension as remove_extension,
    soft_assertions as soft_assertions,
    soft_fail as soft_fail,
)
from .file import contents_of as contents_of



import time
from _typeshed import FileDescriptorOrPath, Incomplete
from collections.abc import Callable, Iterable, Mapping
from logging import Logger
from typing import Any, Final, TypeVar

from .context import Context
from .emitters.udp_emitter import UDPEmitter
from .models.default_dynamic_naming import DefaultDynamicNaming
from .models.dummy_entities import DummySegment, DummySubsegment
from .models.segment import Segment, SegmentContextManager
from .models.subsegment import Subsegment, SubsegmentContextManager
from .sampling.local.sampler import LocalSampler
from .sampling.sampler import DefaultSampler
from .streaming.default_streaming import DefaultStreaming

log: Logger
TRACING_NAME_KEY: Final = "AWS_XRAY_TRACING_NAME"
DAEMON_ADDR_KEY: Final = "AWS_XRAY_DAEMON_ADDRESS"
CONTEXT_MISSING_KEY: Final = "AWS_XRAY_CONTEXT_MISSING"
XRAY_META: Final[dict[str, dict[str, str]]]
SERVICE_INFO: Final[dict[str, str]]

_T = TypeVar("_T")

class AWSXRayRecorder:
    def __init__(self) -> None: ...
    def configure(
        self,
        sampling: bool | None = None,
        plugins: Iterable[str] | None = None,
        context_missing: str | None = None,
        sampling_rules: dict[str, Any] | FileDescriptorOrPath | None = None,
        daemon_address: str | None = None,
        service: str | None = None,
        context: Context | None = None,
        emitter: UDPEmitter | None = None,
        streaming: DefaultStreaming | None = None,
        dynamic_naming: DefaultDynamicNaming | None = None,
        streaming_threshold: int | None = None,
        max_trace_back: int | None = None,
        sampler: LocalSampler | DefaultSampler | None = None,
        stream_sql: bool | None = True,
    ) -> None: ...
    def in_segment(
        self, name: str | None = None, *, traceid: str | None = None, parent_id: str | None = None, sampling: bool | None = None
    ) -> SegmentContextManager: ...
    def in_subsegment(self, name: str | None = None, *, namespace: str = "local") -> SubsegmentContextManager: ...
    def begin_segment(
        self, name: str | None = None, traceid: str | None = None, parent_id: str | None = None, sampling: bool | None = None
    ) -> Segment | DummySegment: ...
    def end_segment(self, end_time: time.struct_time | None = None) -> None: ...
    def current_segment(self) -> Segment: ...
    def begin_subsegment(self, name: str, namespace: str = "local") -> DummySubsegment | Subsegment | None: ...
    def begin_subsegment_without_sampling(self, name: str) -> DummySubsegment | Subsegment | None: ...
    def current_subsegment(self) -> Subsegment | DummySubsegment | None: ...
    def end_subsegment(self, end_time: time.struct_time | None = None) -> None: ...
    def put_annotation(self, key: str, value: Any) -> None: ...
    def put_metadata(self, key: str, value: Any, namespace: str = "default") -> None: ...
    def is_sampled(self) -> bool: ...
    def get_trace_entity(self) -> Segment | Subsegment | DummySegment | DummySubsegment: ...
    def set_trace_entity(self, trace_entity: Segment | Subsegment | DummySegment | DummySubsegment) -> None: ...
    def clear_trace_entities(self) -> None: ...
    def stream_subsegments(self) -> None: ...
    def capture(self, name: str | None = None) -> SubsegmentContextManager: ...
    def record_subsegment(
        self,
        wrapped: Callable[..., _T],
        instance: Any,
        args: Iterable[Incomplete],
        kwargs: Mapping[str, Incomplete],
        name: str,
        namespace: str,
        meta_processor: Callable[..., object] | None,
    ) -> _T: ...
    @property
    def enabled(self) -> bool: ...
    @enabled.setter
    def enabled(self, value: bool) -> None: ...
    @property
    def sampling(self) -> bool: ...
    @sampling.setter
    def sampling(self, value: bool) -> None: ...
    @property
    def sampler(self) -> LocalSampler | DefaultSampler: ...
    @sampler.setter
    def sampler(self, value: LocalSampler | DefaultSampler) -> None: ...
    @property
    def service(self) -> str: ...
    @service.setter
    def service(self, value: str) -> None: ...
    @property
    def dynamic_naming(self) -> DefaultDynamicNaming | None: ...
    @dynamic_naming.setter
    def dynamic_naming(self, value: DefaultDynamicNaming | str) -> None: ...
    @property
    def context(self) -> Context: ...
    @context.setter
    def context(self, cxt: Context) -> None: ...
    @property
    def emitter(self) -> UDPEmitter: ...
    @emitter.setter
    def emitter(self, value: UDPEmitter) -> None: ...
    @property
    def streaming(self) -> DefaultStreaming: ...
    @streaming.setter
    def streaming(self, value: DefaultStreaming) -> None: ...
    @property
    def streaming_threshold(self) -> int: ...
    @streaming_threshold.setter
    def streaming_threshold(self, value: int) -> None: ...
    @property
    def max_trace_back(self) -> int: ...
    @max_trace_back.setter
    def max_trace_back(self, value: int) -> None: ...
    @property
    def stream_sql(self) -> bool: ...
    @stream_sql.setter
    def stream_sql(self, value: bool) -> None: ...



from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: str

class Kubernetes(VaultApiBase):
    def configure(
        self,
        kubernetes_host,
        kubernetes_ca_cert=None,
        token_reviewer_jwt=None,
        pem_keys=None,
        issuer=None,
        mount_point="kubernetes",
        disable_local_ca_jwt: bool = False,
    ): ...
    def read_config(self, mount_point="kubernetes"): ...
    def create_role(
        self,
        name,
        bound_service_account_names,
        bound_service_account_namespaces,
        ttl=None,
        max_ttl=None,
        period=None,
        policies=None,
        token_type: str = "",
        mount_point="kubernetes",
        alias_name_source=None,
        audience: str | None = None,
    ): ...
    def read_role(self, name, mount_point="kubernetes"): ...
    def list_roles(self, mount_point="kubernetes"): ...
    def delete_role(self, name, mount_point="kubernetes"): ...
    def login(self, role, jwt, use_token: bool = True, mount_point="kubernetes"): ...



from typing import Any

class EntryState:
    dn: Any
    status: Any
    attributes: Any
    raw_attributes: Any
    response: Any
    cursor: Any
    origin: Any
    read_time: Any
    changes: Any
    definition: Any
    def __init__(self, dn, cursor) -> None: ...
    def set_status(self, status) -> None: ...
    @property
    def entry_raw_attributes(self): ...

class EntryBase:
    def __init__(self, dn, cursor) -> None: ...
    def __iter__(self): ...
    def __contains__(self, item): ...
    def __getattr__(self, item: str): ...
    def __setattr__(self, item: str, value) -> None: ...
    def __getitem__(self, item): ...
    def __eq__(self, other): ...
    def __lt__(self, other): ...
    @property
    def entry_dn(self): ...
    @property
    def entry_cursor(self): ...
    @property
    def entry_status(self): ...
    @property
    def entry_definition(self): ...
    @property
    def entry_raw_attributes(self): ...
    def entry_raw_attribute(self, name): ...
    @property
    def entry_mandatory_attributes(self): ...
    @property
    def entry_attributes(self): ...
    @property
    def entry_attributes_as_dict(self): ...
    @property
    def entry_read_time(self): ...
    def entry_to_json(
        self,
        raw: bool = False,
        indent: int = 4,
        sort: bool = True,
        stream=None,
        checked_attributes: bool = True,
        include_empty: bool = True,
    ): ...
    def entry_to_ldif(self, all_base64: bool = False, line_separator=None, sort_order=None, stream=None): ...

class Entry(EntryBase):
    def entry_writable(
        self, object_def=None, writer_cursor=None, attributes=None, custom_validator=None, auxiliary_class=None
    ): ...

class WritableEntry(EntryBase):
    def __setitem__(self, key, value) -> None: ...
    def __setattr__(self, item: str, value) -> None: ...
    def __getattr__(self, item: str): ...
    @property
    def entry_virtual_attributes(self): ...
    def entry_commit_changes(self, refresh: bool = True, controls=None, clear_history: bool = True): ...
    def entry_discard_changes(self) -> None: ...
    def entry_delete(self) -> None: ...
    def entry_refresh(self, tries: int = 4, seconds: int = 2): ...
    def entry_move(self, destination_dn) -> None: ...
    def entry_rename(self, new_name) -> None: ...
    @property
    def entry_changes(self): ...



from logging import Logger
from typing_extensions import TypeAlias

from authlib.oauth2.rfc6749 import BaseGrant, TokenEndpointMixin, TokenMixin

_ServerResponse: TypeAlias = tuple[int, str, list[tuple[str, str]]]

log: Logger

class RefreshTokenGrant(BaseGrant, TokenEndpointMixin):
    GRANT_TYPE: str
    INCLUDE_NEW_REFRESH_TOKEN: bool
    def validate_token_request(self) -> None: ...
    def create_token_response(self) -> _ServerResponse: ...
    def issue_token(self, user, refresh_token: TokenMixin) -> dict[str, str | int]: ...
    def authenticate_refresh_token(self, refresh_token: str) -> TokenMixin: ...
    def authenticate_user(self, refresh_token): ...
    def revoke_old_credential(self, refresh_token: TokenMixin) -> None: ...



from _typeshed import Incomplete

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["from_sparse6_bytes", "read_sparse6", "to_sparse6_bytes", "write_sparse6"]

@_dispatchable
def from_sparse6_bytes(string) -> Graph[Incomplete]: ...
def to_sparse6_bytes(G: Graph[_Node], nodes=None, header: bool = True): ...
@_dispatchable
def read_sparse6(path): ...
def write_sparse6(G: Graph[_Node], path, nodes=None, header: bool = True) -> None: ...



from collections.abc import Callable, Sequence
from typing import Final, TypeVar

from tensorflow import Tensor
from tensorflow._aliases import TensorCompatible
from tensorflow.data import Dataset

AUTOTUNE: Final = -1
INFINITE_CARDINALITY: Final = -1
SHARD_HINT: Final = -1
UNKNOWN_CARDINALITY: Final = -2

_T1 = TypeVar("_T1")
_T2 = TypeVar("_T2")

def parallel_interleave(
    map_func: Callable[[_T1], Dataset[_T2]],
    cycle_length: int,
    block_length: int = 1,
    sloppy: bool | None = False,
    buffer_output_elements: int | None = None,
    prefetch_input_elements: int | None = None,
) -> Callable[[Dataset[_T1]], Dataset[_T2]]: ...
def enable_debug_mode() -> None: ...
def cardinality(dataset: Dataset[object]) -> Tensor: ...
def sample_from_datasets(
    datasets: Sequence[Dataset[_T1]],
    weights: TensorCompatible | None = None,
    seed: int | None = None,
    stop_on_empty_dataset: bool = False,
) -> Dataset[_T1]: ...
def __getattr__(name: str): ...  # incomplete module



import sys

from serial.tools.list_ports_common import ListPortInfo

if sys.platform != "win32":
    if sys.platform == "linux":
        from serial.tools.list_ports_linux import comports as comports
    elif sys.platform == "darwin":
        from serial.tools.list_ports_osx import comports as comports
    else:
        def comports(include_links: bool = ...) -> list[ListPortInfo]: ...



from _typeshed import Incomplete

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["second_order_centrality"]

@_dispatchable
def second_order_centrality(G: Graph[_Node], weight: str | None = "weight") -> dict[Incomplete, float]: ...



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["ego_graph"]

@_dispatchable
def ego_graph(G: Graph[_Node], n, radius: float = 1, center: bool = True, undirected: bool = False, distance=None): ...



from _typeshed import Incomplete
from collections.abc import Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["metric_closure", "steiner_tree"]

@_dispatchable
def metric_closure(G: Graph[_Node], weight="weight"): ...
@_dispatchable
def steiner_tree(G: Graph[_Node], terminal_nodes: Iterable[Incomplete], weight: str = "weight", method: str | None = None): ...



from _typeshed import Incomplete

from braintree.error_result import ErrorResult
from braintree.resource_collection import ResourceCollection
from braintree.subscription import Subscription
from braintree.successful_result import SuccessfulResult

class SubscriptionGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def cancel(self, subscription_id: str) -> SuccessfulResult | ErrorResult | None: ...
    def create(self, params: dict[str, Incomplete] | None = None) -> SuccessfulResult | ErrorResult | None: ...
    def find(self, subscription_id: str) -> Subscription: ...
    def retry_charge(self, subscription_id, amount=None, submit_for_settlement: bool = False): ...
    def search(self, *query) -> ResourceCollection: ...
    def update(
        self, subscription_id: str, params: dict[str, Incomplete] | None = None
    ) -> SuccessfulResult | ErrorResult | None: ...



import pycountry.db

LOCALES_DIR: str
DATABASE_DIR: str
__version__: str | None

def resource_filename(package_or_requirement: str, resource_name: str) -> str: ...
def get_version(distribution_name: str) -> str | None: ...
def remove_accents(input_str: str) -> str: ...

class ExistingCountries(pycountry.db.Database):
    data_class: type
    root_key: str
    def search_fuzzy(self, query: str) -> list[pycountry.db.Country]: ...

class HistoricCountries(ExistingCountries):
    data_class: type
    root_key: str

class Scripts(pycountry.db.Database):
    data_class: str
    root_key: str

class Currencies(pycountry.db.Database):
    data_class: str
    root_key: str

class Languages(pycountry.db.Database):
    no_index: list[str]
    data_class: str
    root_key: str

class LanguageFamilies(pycountry.db.Database):
    data_class: str
    root_key: str

class SubdivisionHierarchy(pycountry.db.Data):
    country_code: str
    parent_code: str | None
    def __init__(self, **kw: str) -> None: ...
    @property
    def country(self) -> pycountry.db.Country | None: ...
    @property
    def parent(self) -> SubdivisionHierarchy | None: ...

class Subdivisions(pycountry.db.Database):
    data_class: type
    no_index: list[str]
    root_key: str
    def get(  # type: ignore[override]
        self, *, default: SubdivisionHierarchy | None = ..., **kw: str
    ) -> SubdivisionHierarchy | None | list[SubdivisionHierarchy]: ...
    def match(self, query: str) -> list[SubdivisionHierarchy]: ...
    def partial_match(self, query: str) -> list[SubdivisionHierarchy]: ...
    def search_fuzzy(self, query: str) -> list[SubdivisionHierarchy]: ...

countries: ExistingCountries
subdivisions: Subdivisions
historic_countries: HistoricCountries
currencies: Currencies
languages: Languages
language_families: LanguageFamilies
scripts: Scripts
